﻿using System;
using System.Linq;
using System.Web;
using System.Web.SessionState;
using KRO.Contents;
using KRO.Models;

namespace KRO.Authentication
{
	/// <summary>
	/// Authentication manager
	/// </summary>
	public static class AuthenticationManager
    {        
        /// <summary>
        /// проверка прав доступа к форме, применяется при старте MasterPage, 
        /// при отсутствии прав пользователь перенаправляется на страницу логирования Login
        /// </summary>
        /// <returns>
        /// bool - наличие у пользователя доступа к данной форме
        /// </returns>
        public static bool IsPageProtectedChecked(User AActiveUser,string AFormUrl)
        {   
            // сначала определим общедоступные формы
            if ((AFormUrl.Contains(Constants.LOGIN_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase)) ||
                (AFormUrl.Contains(Constants.ERROR404_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase)) ||
                (AFormUrl.Contains(Constants.ERROR504_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase)))
            { return false; }
            // далее отсеим случай когда пользователь отвалился или просто ссылку кинули
            if ((AActiveUser == null) || (AActiveUser.ID <= 0)) { return true; } 
            else
            { //здесь все формы доступные всем зарегистрированным пользователям
              if ((AFormUrl.Contains(Constants.NAVIGATION_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase)) ||
                 (AFormUrl.Contains(Constants.CHANGE_USER_PASSWORD_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase))) { return false; }
            }
            // далее определимся с доступам к формам по ролям пользователя потом у каждой роли можно определить сочетание флагов
            if ((AFormUrl.Contains(Constants.CHANGE_USER_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.USERS_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.MANAGER_USERS_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_MANAGER_USER_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.LOGOUTS_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.DEPARTMENT_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_DEPARTMENT_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.DEPINCLUDEREGION_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_DEPINCLUDEREGION_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.DEPLISTINPERIOD_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_DEPLISTINPERIOD_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.METRICLISTINPERIOD_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_METRICLISTINPERIOD_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_METRICFUNCTIONINPERIOD_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.REGION_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_REGION_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.METRIC_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_METRIC_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase))
                  && (AuthenticationManager.IsAdmin(AActiveUser))) // только администраторы
            { return false; }
            if ((AFormUrl.Contains(Constants.METRICS_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_REPORTS_PARAMETER_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.GET_REPORT_PARAMETER_FOR_RKCU_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.CHANGE_REPORTPARAMETERBLOCK_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                 AFormUrl.Contains(Constants.REPORTS_PARAMETER_LIST_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase))
                  && (AuthenticationManager.IsUser(AActiveUser))) // остальные пользователи
            { return false; }
            //IsUser
            //Пример отслеживания защищенных форм доступных пользователям с определенными сочетаниями флагов
            /* if (AActiveUser.UserFlag > 0)
             { //пока разграничение пользователей происходит только по ролям (код оставлен для примера)
                 if ((AActiveUser.UserFlag & KROFlag.UserCheckRule) != KROFlag.UserCheckRule) 
                 { //формы пользователям для редактирования соответствующих прав
                     if (AFormUrl.Contains(Constants.CHANGE_MANAGER_USER_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase) ||
                      AFormUrl.Contains(Constants.USERS_PAGE_NAME, StringComparison.InvariantCultureIgnoreCase))
                     { return false; }
                 }
             } else {//в этой ветке можно отследить доступность форм по ролям пользователей
                 // только для Админов

             }*/
            return true; //по умолчанию все формы защищены и никому недоступны
        }
		/// <summary>
		/// Login's user and add's autentication session
		/// </summary>
		/// <param name="response">current user response</param>
		/// <param name="userName">user name</param>
		/// <param name="password">user password</param>
		/// <returns>is login success</returns>
		public static bool LoginUser(HttpSessionState ASession, string userName, string password, out User AUser)
		{
            bool isLoginSuccess = IsUserAccountExists(userName, password, out AUser);

			if (isLoginSuccess)
			{
                AddAuthenticationUserInSession(ASession, AUser);
            }
            else     //сохраним Ошибку входа сессии
                ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgErrorNewSession + " | " + userName + " | " + password, 7), KROFlag.flLogUserSession, -1);
        
			return isLoginSuccess;
		}

		/// <summary>
		/// Logout user and removes authentication cookie
		/// </summary>
		/// <param name="request">current user request</param>
		/// <param name="response">current user response</param>
		public static void LogoutUser(HttpResponse response, HttpSessionState ASession)
        {
            //сохраним сведения о выходе из системы
            User lUser = GetAuthenticationUserModel(ASession);
            ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgOutSession + " " + lUser.Name, 8), KROFlag.flLogUserSession, lUser.ID);
            //очистим данные сессиии
            ASession[KROConstants.SESSION_ACTIVE_USER_MODEL] = new User();
            response.Redirect(Constants.LOGIN_PAGE_URL);
		}

		/// <summary>
		/// Check is user aunthenticated in session
		/// </summary>
		/// <param name="request">current user response</param>
		/// <returns>is user aunthenticated</returns>
        public static bool IsUserAuthenticated(HttpSessionState ASession, out User AUser)
        {  
			bool isAuthenticated = false;

            AUser = GetAuthenticationUserModel(ASession);
			if (AUser != null && AUser.ID > 0)
			{
				isAuthenticated = true;
			}

			return isAuthenticated;
		}

        public static bool IsAdmin(User AActiveUser)
		{
            return AActiveUser.UserRoleName == KROConstants.ADMIN_ROLE_NAME;
		}
        public static bool IsUser(User AActiveUser)
        {
            return AActiveUser.UserRoleName == KROConstants.ADMIN_ROLE_USER;
        }
        private static bool IsUserAccountExists(string userName, string password, out User AUser)
		{
            bool lResult = false;
            AUser = UsersManager.GetOnlyOneUser(userName, password);
            if (AUser != null) lResult = true;
            return lResult;
		}

        public static void AddAuthenticationUserInSession(HttpSessionState ASession, User AUser)
        {
            ASession[KROConstants.SESSION_ACTIVE_USER_MODEL] = AUser;
            //сохраним данные создания сессии
            ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgNewSession + " " + AUser.Name, 7), KROFlag.flLogUserSession, AUser.ID);
        }
        public static User GetAuthenticationUserModel(HttpSessionState ASession)
        {
            User lUser = new User();
            if ((ASession != null) && (ASession.Count > 0))
            {
                try
                {   //попробуем получить пользователя сессии
                    lUser = (User)ASession[KROConstants.SESSION_ACTIVE_USER_MODEL];
                }
                catch { }
            }
            return lUser;
        }
	}
}