﻿using System;
using KRO.Contents;
using System.Globalization;

namespace KRO.Authentication
{
	/// <summary>
	/// User entity
	/// </summary> 
    [Serializable]
	public class User
	{
		public int ID { get; private set; }
        public string Family { get; set; } 
		public string FirstName { get; set; } 
		public string SecondName { get; set; }
		public string Name { get; set; }
		public string Password { get; set; }
		public int UserRoleID  { get;set; }
        public string UserRoleName { get; set; }
        public int UserDepartmentID { get; set; }
        public string UserDepartmentName { get; set; }        
        public string OfficeName { get; set; }
        public string EMail { get; set; }
        public string Phone { get; set; }
		public long UserFlag  { get; set; }
        public DateTime DatePassword { get; set; }
        public bool IsDeleted { get; set; }
        //Flag Fields
        public bool CheckAutoPassword { get; private set; }
        public bool CheckAddNewReportParameter { get; private set; }
        public bool CheckRKCU { get; private set; }
        public bool CheckEconomicDepartment { get; private set; }
        public bool CheckFilialDepartment { get; private set; }
        public bool CheckRegionDirection { get; private set; }
        

        public User(int AID, string AFamily, string AFirstName, string ASecondName, string AName, string APassword,
                    int ARoleID, string ARoleName, int ADepartmentID, string ADepartmentName,
                    string AOfficeName, string AEMail, string APhone, long AFlag, DateTime ADatePassword, bool AIsDeleted = false)
		{
            this.ID = AID;
            this.Family = AFamily;
            this.FirstName = AFirstName;
            this.SecondName = ASecondName;
            this.Name = AName;
            this.Password = APassword; 
            this.UserRoleID = ARoleID;
            this.UserRoleName = ARoleName;
            this.UserDepartmentID = ADepartmentID;
            this.UserDepartmentName = ADepartmentName;
            this.OfficeName = AOfficeName;
            this.EMail= AEMail;
            this.Phone= APhone;
            this.UserFlag = AFlag;
            this.DatePassword = ADatePassword;
            this.IsDeleted = AIsDeleted;
            SetFlagFields();
		}
        public User(string AFamily, string AFirstName, string ASecondName, string AName, string APassword,
                   int ARoleID, string ARoleName, int ADepartmentID, string ADepartmentName,
                   string AOfficeName, string AEMail, string APhone, long AFlag, DateTime ADatePassword, bool AIsDeleted = false)
        {
            this.ID = -1;
            this.Family = AFamily;
            this.FirstName = AFirstName;
            this.SecondName = ASecondName;
            this.Name = AName;
            this.Password = APassword;
            this.UserRoleID = ARoleID;
            this.UserRoleName = ARoleName;
            this.UserDepartmentID = ADepartmentID;
            this.UserDepartmentName = ADepartmentName;
            this.OfficeName = AOfficeName;
            this.EMail= AEMail;
            this.Phone= APhone;
            this.UserFlag = AFlag;
            this.DatePassword = ADatePassword;
            this.IsDeleted = AIsDeleted;
            SetFlagFields();
        }
        public User()
        {
            this.ID = -1;
            this.Family = "";
            this.FirstName = "";
            this.SecondName = "";
            this.Name = "";
            this.Password = "";
            this.UserRoleID = -1;
            this.UserRoleName = "";
            this.UserDepartmentID = -1;
            this.UserDepartmentName = "";
            this.OfficeName = "";
            this.EMail= "";
            this.Phone= ""; 
            this.UserFlag = 0;
            this.DatePassword = DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture);
            this.IsDeleted = false;
            SetFlagFields();
        }
        public void SetUserID(int AUserID)
        {
            if (AUserID > 0) this.ID = AUserID; else this.ID = -1; 
        }
        private void SetFlagFields()
        {
          if ((this.UserFlag & KROFlag.flUserCheckAutoPassword) == KROFlag.flUserCheckAutoPassword) this.CheckAutoPassword = true; else this.CheckAutoPassword = false;
          if ((this.UserFlag & KROFlag.flUserAddNewReportParameter) == KROFlag.flUserAddNewReportParameter) this.CheckAddNewReportParameter = true; else this.CheckAddNewReportParameter = false;
          if ((this.UserFlag & KROFlag.flUserRCKU) == KROFlag.flUserRCKU) this.CheckRKCU = true; else this.CheckRKCU = false;
          if ((this.UserFlag & KROFlag.flUserEconomicDepartment) == KROFlag.flUserEconomicDepartment) this.CheckEconomicDepartment = true; else this.CheckEconomicDepartment = false;
          if ((this.UserFlag & KROFlag.flUserFilialDepartment) == KROFlag.flUserFilialDepartment) this.CheckFilialDepartment = true; else this.CheckFilialDepartment = false;
          if ((this.UserFlag & KROFlag.flUserRegionDirection) == KROFlag.flUserRegionDirection) this.CheckRegionDirection = true; else this.CheckRegionDirection = false;
        }
	}
}