﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Globalization;
using System.Web.SessionState;
using KRO.Contents;
using KRO.Models;

namespace KRO.Authentication
{
	/// <summary>
	/// Manager for User entities
	/// </summary>
	public static class UsersManager
	{
        public static User GetOnlyOneUser(string ALogin, string APassword)
        {   
            User lUser = null; 
            using (
                 SqlConnection connection =
                     new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try 
                {
                    SqlCommand selectOnlyOneUserCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ONLY_ONE_USER_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    selectOnlyOneUserCommand.Parameters.AddWithValue("@Login", ALogin);
                    selectOnlyOneUserCommand.Parameters.AddWithValue("@Password", APassword);

                    connection.Open();

                    using (SqlDataReader lUsersReader = selectOnlyOneUserCommand.ExecuteReader())
                    {
                        if ((lUsersReader.HasRows) && (lUsersReader.Read())) {
                           lUser = new User(int.Parse(lUsersReader["UserID"].ToString()),
                                                      lUsersReader["Family"] == DBNull.Value ? string.Empty : (string)lUsersReader["Family"],
                                                      lUsersReader["FirstName"] == DBNull.Value ? string.Empty : (string)lUsersReader["FirstName"],
                                                      lUsersReader["SecondName"] == DBNull.Value ? string.Empty : (string)lUsersReader["SecondName"],
                                                      lUsersReader["UserName"] == DBNull.Value ? string.Empty : (string)lUsersReader["UserName"],
                                                      lUsersReader["UserPassword"] == DBNull.Value ? null : (string)lUsersReader["UserPassword"],
                                                      lUsersReader["UserRoleID"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserRoleID"].ToString()),
                                                      lUsersReader["RoleName"] == DBNull.Value ? string.Empty : (string)lUsersReader["RoleName"],
                                                      lUsersReader["UserDepartmentID"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserDepartmentID"].ToString()),
                                                      lUsersReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)lUsersReader["DepartmentName"],
                                                      lUsersReader["OfficeName"] == DBNull.Value ? string.Empty : (string)lUsersReader["OfficeName"],
                                                      lUsersReader["EMail"] == DBNull.Value ? string.Empty : (string)lUsersReader["EMail"],
                                                      lUsersReader["Phone"] == DBNull.Value ? string.Empty : (string)lUsersReader["Phone"],
                                                      lUsersReader["UserFlag"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserFlag"].ToString()),
                                                      lUsersReader["Datepassword"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(lUsersReader["Datepassword"].ToString()),
                                                      lUsersReader["IsDeleted"] == DBNull.Value ? false : (bool) lUsersReader["IsDeleted"]
                                            );
                            }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lUser;
        }
		public static List<User> GetUsers(int AType,int AUserID, bool AWithoutAdmin = false)
		{
			List<User> lUsers = new List<User>();

			using (
				SqlConnection connection =
					new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
			{
                try
                {
                    SqlCommand selectAllUsersCommand;

                    if (AType == KROConstants.FORM_USERS_FOR_ADMIN)
                    {
                        selectAllUsersCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_USERS_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                    }
                    else
                    {
                        selectAllUsersCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_MANAGER_USERS_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        selectAllUsersCommand.Parameters.AddWithValue("@UserID", AUserID);
                    }
                    connection.Open();

                    using (SqlDataReader lUsersReader = selectAllUsersCommand.ExecuteReader())
                    {
                        while (lUsersReader.Read())
                        {
                            if (AWithoutAdmin)
                            {
                                if (int.Parse(lUsersReader["UserID"].ToString()) != 1)
                                {
                                    User lUser = new User(int.Parse(lUsersReader["UserID"].ToString()),
                                                          lUsersReader["Family"] == DBNull.Value ? string.Empty : (string)lUsersReader["Family"],
                                                          lUsersReader["FirstName"] == DBNull.Value ? string.Empty : (string)lUsersReader["FirstName"],
                                                          lUsersReader["SecondName"] == DBNull.Value ? string.Empty : (string)lUsersReader["SecondName"],
                                                          lUsersReader["UserName"] == DBNull.Value ? string.Empty : (string)lUsersReader["UserName"],
                                                          lUsersReader["UserPassword"] == DBNull.Value ? null : (string)lUsersReader["UserPassword"],
                                                          lUsersReader["UserRoleID"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserRoleID"].ToString()),
                                                          lUsersReader["RoleName"] == DBNull.Value ? string.Empty : (string)lUsersReader["RoleName"],
                                                          lUsersReader["UserDepartmentID"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserDepartmentID"].ToString()),
                                                          lUsersReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)lUsersReader["DepartmentName"],
                                                          lUsersReader["OfficeName"] == DBNull.Value ? string.Empty : (string)lUsersReader["OfficeName"],
                                                          lUsersReader["EMail"] == DBNull.Value ? string.Empty : (string)lUsersReader["EMail"],
                                                          lUsersReader["Phone"] == DBNull.Value ? string.Empty : (string)lUsersReader["Phone"],
                                                          lUsersReader["UserFlag"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserFlag"].ToString()),
                                                          lUsersReader["Datepassword"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(lUsersReader["Datepassword"].ToString()),
                                                          lUsersReader["IsDeleted"] == DBNull.Value ? false : (bool)lUsersReader["IsDeleted"]
                                                );
                                    lUsers.Add(lUser);
                                }
                            }
                            else
                            {
                                User lUser = new User(int.Parse(lUsersReader["UserID"].ToString()),
                                                                                  lUsersReader["Family"] == DBNull.Value ? string.Empty : (string)lUsersReader["Family"],
                                                                                  lUsersReader["FirstName"] == DBNull.Value ? string.Empty : (string)lUsersReader["FirstName"],
                                                                                  lUsersReader["SecondName"] == DBNull.Value ? string.Empty : (string)lUsersReader["SecondName"],
                                                                                  lUsersReader["UserName"] == DBNull.Value ? string.Empty : (string)lUsersReader["UserName"],
                                                                                  lUsersReader["UserPassword"] == DBNull.Value ? null : (string)lUsersReader["UserPassword"],
                                                                                  lUsersReader["UserRoleID"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserRoleID"].ToString()),
                                                                                  lUsersReader["RoleName"] == DBNull.Value ? string.Empty : (string)lUsersReader["RoleName"],
                                                                                  lUsersReader["UserDepartmentID"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UseDepartmentID"].ToString()),
                                                                                  lUsersReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)lUsersReader["DepartmentnName"],
                                                                                  lUsersReader["OfficeName"] == DBNull.Value ? string.Empty : (string)lUsersReader["OfficeName"],
                                                                                  lUsersReader["EMail"] == DBNull.Value ? string.Empty : (string)lUsersReader["EMail"],
                                                                                  lUsersReader["Phone"] == DBNull.Value ? string.Empty : (string)lUsersReader["Phone"],
                                                                                  lUsersReader["UserFlag"] == DBNull.Value ? 0 : (int)int.Parse(lUsersReader["UserFlag"].ToString()),
                                                                                  lUsersReader["Datepassword"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(lUsersReader["Datepassword"].ToString()),
                                                                                  lUsersReader["IsDeleted"] == DBNull.Value ? false : (bool)lUsersReader["IsDeleted"]
                                                      );
                                lUsers.Add(lUser);
                            }
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
			}
            return lUsers;
		}
		public static bool DeleteUser(int AUserID, bool IsDeleted, HttpSessionState ASession)
		{
            bool lResult = false;
            string lQuery;
            CommandType lCommandType;
            if (IsDeleted) lQuery = KROSQLs.DELETE_USER_SP_NAME; else lQuery = KROSQLs.NOT_DELETE_USER_QUERY;
            if (IsDeleted) lCommandType = CommandType.StoredProcedure; else lCommandType = CommandType.Text;
			using (
				SqlConnection connection =
					new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
			{
                try
                { 
				    SqlCommand deleteUserCommand = new SqlCommand
				    {
					    Connection = connection,
                        CommandText = lQuery,
                        CommandType = lCommandType
				    };

                    deleteUserCommand.Parameters.Add(new SqlParameter("UserID", AUserID));

				    connection.Open();

				    deleteUserCommand.ExecuteNonQuery();
                    lResult = true;
                }
                finally { if (connection != null) connection.Close(); }
            }

            //сохраним системное сообщение
            if (lResult) ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgBlockUser + " | UserID = " + AUserID.ToString(), 3), KROFlag.flLogExchangeUsers, AuthenticationManager.GetAuthenticationUserModel(ASession).ID);
            else ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgBlockUser + " | " + " | UserID = " + AUserID.ToString(), 2), KROFlag.flLogExchangeUsers, AuthenticationManager.GetAuthenticationUserModel(ASession).ID);
           
            return lResult;
		}

		public static bool InsertUser(User AUser, bool AStringForeastKey = false)
		{   
            bool lResult = false;
            if ((AUser.Family.Length>0) &&
                (AUser.Name.Length>0) &&
                (AUser.Password.Length>0) &&
                (AUser.UserFlag >= 0) &&
                ((AStringForeastKey && (AUser.UserRoleName.Length > 0) && (AUser.UserDepartmentName.Length > 0)) ||
                (!AStringForeastKey && (AUser.UserDepartmentID > 0) && (AUser.UserRoleID > 0))))
            {
			    using (
				    SqlConnection connection =
					    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
			    {       
                    try 
                    {
                        //определим тип процедуры insert user
                        string lInsertProcedureName;
                        if (AStringForeastKey) { lInsertProcedureName = KROSQLs.INSERT_USER_STRING_FORENT_KEY; } else { lInsertProcedureName = KROSQLs.INSERT_USER_INTEGER_FORENT_KEY;} 

				        SqlCommand insertUserCommand = new SqlCommand
				        {
					        Connection = connection,
                            CommandText = lInsertProcedureName,
					        CommandType = CommandType.StoredProcedure
				        };

                        SqlParameter lParameter = null;
                        insertUserCommand.Parameters.Add(new SqlParameter("Family", AUser.Family));
                        //заменим пустые значения на null 
                        if (AUser.FirstName.Length == 0) insertUserCommand.Parameters.Add(new SqlParameter("FirstName", DBNull.Value));
                        else insertUserCommand.Parameters.Add(new SqlParameter("FirstName", AUser.FirstName));
                        if (AUser.SecondName.Length == 0) insertUserCommand.Parameters.Add(new SqlParameter("SecondName", DBNull.Value));
                        else insertUserCommand.Parameters.Add(new SqlParameter("SecondName", AUser.SecondName));
                        if (AUser.OfficeName.Length == 0) insertUserCommand.Parameters.Add(new SqlParameter("UserOfficeName", DBNull.Value));
                        else insertUserCommand.Parameters.Add(new SqlParameter("UserOfficeName", AUser.OfficeName));
                        if (AUser.EMail.Length == 0) insertUserCommand.Parameters.Add(new SqlParameter("UserEMail", DBNull.Value));
                        else insertUserCommand.Parameters.Add(new SqlParameter("UserEMail", AUser.EMail));
                        if (AUser.Phone.Length == 0) insertUserCommand.Parameters.Add(new SqlParameter("UserPhone", DBNull.Value));
                        else insertUserCommand.Parameters.Add(new SqlParameter("UserPhone", AUser.Phone));

                        insertUserCommand.Parameters.Add(new SqlParameter("UserName", AUser.Name));
                        insertUserCommand.Parameters.Add(new SqlParameter("UserPassword", AUser.Password));
                        // в зависимости от типа процедуры передаем соответствующие параметры
                        if (AStringForeastKey) 
                        {
                            insertUserCommand.Parameters.Add(new SqlParameter("UserRoleName", AUser.UserRoleName));
                            insertUserCommand.Parameters.Add(new SqlParameter("UserDepartmentName", AUser.UserDepartmentName));
                        }
                        else
                        {
                            lParameter = new SqlParameter("@UserRoleID", SqlDbType.Int);
                            lParameter.IsNullable = false;
                            lParameter.Value = AUser.UserRoleID;
                            insertUserCommand.Parameters.Add(lParameter);
                            lParameter = new SqlParameter("@UserDepartmentID", SqlDbType.Int);
                            lParameter.IsNullable = false;
                            lParameter.Value = AUser.UserDepartmentID;
                            insertUserCommand.Parameters.Add(lParameter);
                        }
                        lParameter = new SqlParameter("@UserFlag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUser.UserFlag;
                        insertUserCommand.Parameters.Add(lParameter);
                        lParameter = new SqlParameter("@DatePassword", SqlDbType.Date);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUser.DatePassword;
                        insertUserCommand.Parameters.Add(lParameter);
				        connection.Open();
                        AUser.SetUserID((int)insertUserCommand.ExecuteScalar());
                        lResult = true;
                    } finally { if (connection != null) connection.Close(); }
			    }
            }
            return lResult; 
		}
        public static bool UpdateUser(User AUser, bool AStringForeastKey = false)
		{
            bool lResult = false;
            if ((AUser.ID > 0) &&
                (AUser.Family.Length > 0) &&
                (AUser.Name.Length > 0) &&
                (AUser.Password.Length > 0) &&
                (AUser.UserFlag >= 0) &&
                ((AStringForeastKey && (AUser.UserRoleName.Length > 0) && (AUser.UserDepartmentName.Length > 0)) ||
                (!AStringForeastKey && (AUser.UserDepartmentID > 0) && (AUser.UserRoleID > 0))))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        //определим тип процедуры insert user
                        string lUpdateProcedureName;
                        if (AStringForeastKey) { lUpdateProcedureName = KROSQLs.UPDATE_USER_STRING_FORENT_KEY; } else { lUpdateProcedureName = KROSQLs.UPDATE_USER_INTEGER_FORENT_KEY; }

                        SqlCommand updateUserCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = lUpdateProcedureName,
                            CommandType = CommandType.StoredProcedure
                        };
                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUser.ID;
                        updateUserCommand.Parameters.Add(lParameter);
                        updateUserCommand.Parameters.Add(new SqlParameter("Family", AUser.Family));
                        //заменим пустые значения на null 
                        if (AUser.FirstName.Length == 0)
                        {
                            lParameter = new SqlParameter("@FirstName", SqlDbType.NVarChar);
                            lParameter.IsNullable = true;
                            lParameter.Value = DBNull.Value;
                            updateUserCommand.Parameters.Add(lParameter);
                        }
                        else updateUserCommand.Parameters.Add(new SqlParameter("FirstName", AUser.FirstName));
                        if (AUser.SecondName.Length == 0)
                        {
                            lParameter = new SqlParameter("@SecondName", SqlDbType.NVarChar);
                            lParameter.IsNullable = true;
                            lParameter.Value = DBNull.Value;
                            updateUserCommand.Parameters.Add(lParameter);
                        }
                        else updateUserCommand.Parameters.Add(new SqlParameter("SecondName", AUser.SecondName));
                        if (AUser.OfficeName.Length == 0)
                        {
                            lParameter = new SqlParameter("@UserOfficeName", SqlDbType.NVarChar);
                            lParameter.IsNullable = true;
                            lParameter.Value = DBNull.Value;
                            updateUserCommand.Parameters.Add(lParameter);
                        }
                        else updateUserCommand.Parameters.Add(new SqlParameter("UserOfficeName", AUser.OfficeName));
                        if (AUser.EMail.Length == 0)
                        {
                            lParameter = new SqlParameter("@UserEMail", SqlDbType.NVarChar);
                            lParameter.IsNullable = true;
                            lParameter.Value = DBNull.Value;
                            updateUserCommand.Parameters.Add(lParameter);
                        }
                        else updateUserCommand.Parameters.Add(new SqlParameter("UserEMail", AUser.EMail));
                        if (AUser.Phone.Length == 0)
                        {
                            lParameter = new SqlParameter("@UserPhone", SqlDbType.NVarChar);
                            lParameter.IsNullable = true;
                            lParameter.Value = DBNull.Value;
                            updateUserCommand.Parameters.Add(lParameter);
                        }
                        else updateUserCommand.Parameters.Add(new SqlParameter("UserPhone", AUser.Phone));

                        updateUserCommand.Parameters.Add(new SqlParameter("UserName", AUser.Name));
                        updateUserCommand.Parameters.Add(new SqlParameter("UserPassword", AUser.Password));
                        // в зависимости от типа процедуры передаем соответствующие параметры
                        if (AStringForeastKey)
                        {
                            updateUserCommand.Parameters.Add(new SqlParameter("UserRoleName", AUser.UserRoleName));
                            updateUserCommand.Parameters.Add(new SqlParameter("UserDepartmentName", AUser.UserDepartmentName));
                        }
                        else
                        {
                            lParameter = new SqlParameter("@UserRoleID", SqlDbType.Int);
                            lParameter.IsNullable = false;
                            lParameter.Value = AUser.UserRoleID;
                            updateUserCommand.Parameters.Add(lParameter);
                            lParameter = new SqlParameter("@UserDepartmentID", SqlDbType.Int);
                            lParameter.IsNullable = false;
                            lParameter.Value = AUser.UserDepartmentID;
                            updateUserCommand.Parameters.Add(lParameter);
                        }
                        lParameter = new SqlParameter("@UserFlag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUser.UserFlag;
                        updateUserCommand.Parameters.Add(lParameter);
                        lParameter = new SqlParameter("@DatePassword", SqlDbType.Date);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUser.DatePassword;
                        updateUserCommand.Parameters.Add(lParameter);
                        connection.Open();
                        updateUserCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult; 
		}
        public static bool ChangeUserPassword(User AUser, string ALogin, string AOldPassword, string ANewPassword)
        {
            bool lResult = false;
            if ((AUser.ID > 0)&&
                (ALogin.Length > 0) &&
                (ANewPassword.Length > 0) &&
                (AOldPassword.Length > 0) &&
                (AUser.Name.Equals(ALogin)) &&
                (AUser.Password.Equals(AOldPassword)))
            {
            using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                  {
                      try
                      {
                            SqlCommand lChangeUserPasswordCommand = new SqlCommand
                            {
                                Connection = connection,
                                CommandText = KROSQLs.CHANGE_USER_PASSWORD,
                                CommandType = CommandType.StoredProcedure
                            };

                            SqlParameter lParameter = null;
                            lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                            lParameter.IsNullable = false;
                            lParameter.Value = AUser.ID;
                            lChangeUserPasswordCommand.Parameters.Add(lParameter);

                            lChangeUserPasswordCommand.Parameters.Add(new SqlParameter("UserPassword", ANewPassword));

                            // разберемся с флагами
                            long lFlag = AUser.UserFlag;
                            if ((lFlag & KROFlag.flUserCheckAutoPassword) == KROFlag.flUserCheckAutoPassword)
                            {
                                lFlag &= ~KROFlag.flUserCheckAutoPassword;  
                            }
                            lParameter = new SqlParameter("@UserFlag", SqlDbType.Int);
                            lParameter.IsNullable = false;
                            lParameter.Value = lFlag;
                            lChangeUserPasswordCommand.Parameters.Add(lParameter);
                            connection.Open();                    
				            lChangeUserPasswordCommand.ExecuteNonQuery();
                            lResult = true;
                      }
                      finally { if (connection != null) connection.Close(); }
                 }
            }
            return lResult;
        }
	}
}