﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using KRO.Contents;

namespace KRO.Models
{
    [Serializable]
    public class MetricLine
    {
        public int MetricID { get; private set; }
        public string MetricName { get; set; }
        public string MetricSecondName { get; set; }
        public string MetricSort { get; set; }
        public string MetricCode { get; set; }
        public string MetricScreenCode { get; set; }
        public int MetricFarmID { get; set; }
        public string MetricFarmName { get; set; }
        public string MetricFarmSort { get; set; }
        public int MetricCategoryID { get; set; }
        public string MetricCategoryName { get; set; }
        public long Flag { get; private set; }
        public bool MetricChecked { get; set; } //for only table MetricListInPeriod generate MetricList
        // Flag fields
        public bool FlagMetricWrite { get; private set; }
        public bool FlagMetricCalculation { get; private set; }
        public bool FlagMetricTextRow { get; private set; }
        public string MetricColorCode { get; private set; }
        public MetricLine(int AID, string AMetricName, string AMetricSecondName, string AMetricCode, string AMetricSort, string AMetricScreenCode,
                         int AMetricFarmID, string AMetricFarmName, string AMetricFarmSort, int AMetricCategoryID, string AMetricCategoryName, long AFlag, 
                         bool AMetricChecked = false)
        {
            this.MetricID = AID;
            this.MetricName = AMetricName;
            this.MetricSecondName = AMetricSecondName;
            this.MetricSort = AMetricSort;
            this.MetricCode = AMetricCode;
            this.MetricScreenCode = AMetricScreenCode;
            this.MetricFarmID = AMetricFarmID;
            this.MetricFarmName = AMetricFarmName;
            this.MetricFarmSort = AMetricFarmSort;
            this.MetricCategoryID = AMetricCategoryID;
            this.MetricCategoryName = AMetricCategoryName;
            this.Flag = AFlag;
            this.MetricChecked = AMetricChecked;
            SetFlagFields();
        }
        public MetricLine()
        {
            this.MetricID = -1;
            this.MetricName = "";
            this.MetricSecondName = "";
            this.MetricSort = "";
            this.MetricCode = "";
            this.MetricScreenCode = "";
            this.MetricFarmID = -1;
            this.MetricFarmName = "";
            this.MetricFarmSort = "";
            this.MetricCategoryID = -1;
            this.MetricCategoryName = "";
            this.Flag = 0;
            this.MetricChecked = false;
            SetFlagFields();
        }
        public void SetMetricID(int AMetricID)
        {
            if (AMetricID > 0) this.MetricID = AMetricID; else this.MetricID = -1;
        }
        public void SetFlag(long AFlag)
        {
            if (AFlag > 0) this.Flag= AFlag; else this.Flag = 0;
            SetFlagFields();
        }
        private void SetFlagFields()
        {
            // определим значение по умолчанию для цвета отображения (белый)
            this.MetricColorCode = KROConstants.clRowDefaultCode;
            //далее разбор флага
            if ((this.Flag & KROFlag.flMetricTextRow) == KROFlag.flMetricTextRow)  
            {   // если показатель текстовый то ни вводить значения ни участвовать в расчетах не может
                this.FlagMetricTextRow = true; 
                this.FlagMetricCalculation = false;
                this.FlagMetricWrite = false;
                this.MetricColorCode = KROConstants.clRowGrayCode;
            }
            else 
            {
                this.FlagMetricTextRow = false;
                //дальше начнем выставлять рассчетные показатели
                if ((this.Flag & KROFlag.flMetricWrite) == KROFlag.flMetricWrite) this.FlagMetricWrite = true; else this.FlagMetricWrite = false;
                if ((this.Flag & KROFlag.flMetricCalculations) == KROFlag.flMetricCalculations) this.FlagMetricCalculation = true; else this.FlagMetricCalculation = false;
                // вспомогательные свойства из поля флаг
                // цвет отображения показателя
                //определим цвет строки
                if ((this.Flag & KROFlag.flMetricBrownColor) == KROFlag.flMetricBrownColor) this.MetricColorCode = KROConstants.clRowBrownCode;
                if ((this.Flag & KROFlag.flMetricYellowColor) == KROFlag.flMetricYellowColor) this.MetricColorCode = KROConstants.clRowYellowCode;
                if ((this.Flag & KROFlag.flMetricGreenColor) == KROFlag.flMetricGreenColor) this.MetricColorCode = KROConstants.clRowGreenCode;
                if ((this.Flag & KROFlag.flMetricBlueColor) == KROFlag.flMetricBlueColor) this.MetricColorCode = KROConstants.clRowBlueCode;
            }
        }
    }
}