﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using KRO.Models;
using KRO.Contents;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;
using System.Globalization;
using System.Web.SessionState;
using KRO.Authentication;

namespace KRO.Models
{
    public static class ModelsManager
    {
        # region RoleList
        public static List<RoleLine> GetRolesList()
        {
            List<RoleLine> lRolesList = new List<RoleLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                { 
                    SqlCommand selectAllRolesCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SelectAllRolesSQL,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader rolesReader = selectAllRolesCommand.ExecuteReader())
                    {
                        while (rolesReader.Read())
                        {
                            RoleLine  currentRole = new RoleLine(Convert.ToInt32(rolesReader.GetValue(0)),
                                                                 rolesReader.GetValue(1).ToString(),
                                                                 Convert.ToInt64(rolesReader.GetValue(2).ToString()));
                            lRolesList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRolesList;
        }
        # endregion
        # region RegionList
        public static List<RegionLine> GetRegionList()
        {
            List<RegionLine> lRegionList = new List<RegionLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllRegionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SelectAllRegionSQL,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader RegionReader = selectAllRegionCommand.ExecuteReader())
                    {
                        while (RegionReader.Read())
                        {
                            RegionLine currentRole = new RegionLine(Convert.ToInt32(RegionReader.GetValue(0)),
                                                                    RegionReader.GetValue(1).ToString(),
                                                                    RegionReader.GetValue(2).ToString(),
                                                                    RegionReader.GetValue(3).ToString());
                            lRegionList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRegionList;
        }
        public static List<RegionLine> GetReportRegionList(int AReportParameterID, int AUserID, bool AOnlyWrite = false)
        {
            List<RegionLine> lRegionList = new List<RegionLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    string lSQL;

                    if (!AOnlyWrite) lSQL = KROSQLs.SELECT_ALL_REPORTREGION_BY_REPORTPARAMETERID_AND_USERID_SP_NAME;
                    else lSQL = KROSQLs.SELECT_ALL_REPORTREGION_ONLYWRITE_BY_REPORTPARAMETERID_AND_USERID_SP_NAME;
                        
                    SqlCommand selectAllReportRegionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = lSQL,
                        CommandType = CommandType.StoredProcedure
                    };
                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@ReportsParameterID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportParameterID;
                    selectAllReportRegionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AUserID;
                    selectAllReportRegionCommand.Parameters.Add(lParameter);
                    connection.Open();
                    using (SqlDataReader RegionReader = selectAllReportRegionCommand.ExecuteReader())
                    {
                        while (RegionReader.Read())
                        {
                            RegionLine currentRole = new RegionLine(Convert.ToInt32(RegionReader.GetValue(0)),
                                                                    RegionReader["RegionDirection"] == DBNull.Value ? string.Empty : (string)RegionReader["RegionDirection"],
                                                                    RegionReader["RegionCode"] == DBNull.Value ? string.Empty : (string)RegionReader["RegionCode"],
                                                                    RegionReader["RegionSort"] == DBNull.Value ? string.Empty : (string)RegionReader["RegionSort"],
                                                                    RegionReader["Flag"] == DBNull.Value ? 0 : (long)long.Parse(RegionReader["Flag"].ToString())
                                                                    );
                            lRegionList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRegionList;
        }
        public static List<RegionLine> GetReportBlockRegionList(int AReportParameterID)
        {
            List<RegionLine> lRegionList = new List<RegionLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    string lSQL = KROSQLs.SELECT_ALL_BLOCK_REGION_BY_REPORTPARAMETERID_SP_NAME;

                    SqlCommand selectAllReportBlockRegionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = lSQL,
                        CommandType = CommandType.StoredProcedure
                    };
                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@ReportsParameterID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportParameterID;
                    selectAllReportBlockRegionCommand.Parameters.Add(lParameter);

                    connection.Open();
                    using (SqlDataReader BlockRegionReader = selectAllReportBlockRegionCommand.ExecuteReader())
                    {
                        while (BlockRegionReader.Read())
                        {
                            RegionLine currentRole = new RegionLine(Convert.ToInt32(BlockRegionReader["RegionID"].ToString()),
                                                                    string.Empty,
                                                                    string.Empty,
                                                                    string.Empty,
                                                                    0
                                                                    );
                            lRegionList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRegionList;
        }
        public static List<RegionLine> GetReportUnBlockRegionList(int AReportParameterID)
        {
            List<RegionLine> lRegionList = new List<RegionLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    string lSQL = KROSQLs.SELECT_ALL_UN_BLOCK_REGION_BY_REPORTPARAMETERID_SP_NAME;

                    SqlCommand selectAllReportUnBlockRegionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = lSQL,
                        CommandType = CommandType.StoredProcedure
                    };
                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@ReportsParameterID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportParameterID;
                    selectAllReportUnBlockRegionCommand.Parameters.Add(lParameter);

                    connection.Open();
                    using (SqlDataReader UnBlockRegionReader = selectAllReportUnBlockRegionCommand.ExecuteReader())
                    {
                        while (UnBlockRegionReader.Read())
                        {
                            RegionLine currentRole = new RegionLine(Convert.ToInt32(UnBlockRegionReader["Temp_RegionID"].ToString()),
                                                                    UnBlockRegionReader["RegionDirection"] == DBNull.Value ? string.Empty : UnBlockRegionReader["RegionDirection"].ToString(),
                                                                    string.Empty,
                                                                    string.Empty,
                                                                    0
                                                                    );
                            lRegionList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRegionList;
        }
        public static bool InsertRegion(RegionLine ARegionLine)
        {
            bool lResult = false;

            if ((ARegionLine.RegionName.Length > 0) &&
                (ARegionLine.RegionCode.Length > 0) &&
                (ARegionLine.RegionSort.Length > 0))
            {
                //подготовка данных подрежем первые 10 символов у ARegionCode и ARegionSort и подправим ARegionCategory
                if (ARegionLine.RegionCode.Length > KROConstants.MaxLegthCodeSort) ARegionLine.RegionCode = ARegionLine.RegionCode.Substring(0, KROConstants.MaxLegthCodeSort);
                if (ARegionLine.RegionSort.Length > KROConstants.MaxLegthCodeSort) ARegionLine.RegionSort = ARegionLine.RegionSort.Substring(0, KROConstants.MaxLegthCodeSort);
                //конец подготовки данных
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertRegionCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_REGION_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@RegionName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ARegionLine.RegionName;
                        insertRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@RegionCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ARegionLine.RegionCode;
                        insertRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@RegionSort", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ARegionLine.RegionSort;
                        insertRegionCommand.Parameters.Add(lParameter);

                        connection.Open();
                        ARegionLine.SetRegionID((int)insertRegionCommand.ExecuteScalar());
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateRegionCheck(RegionLine ARegionLine)
        {
            bool lResult = false;
            if ((ARegionLine.RegionID > 0) &&
                (ARegionLine.RegionName.Length > 0) &&
                (ARegionLine.RegionCode.Length > 0) &&
                (ARegionLine.RegionSort.Length > 0))
            {
                //подготовка данных подрежем первые 10 символов у ARegionCode и ARegionSort и подправим ARegionCategory
                if (ARegionLine.RegionCode.Length > KROConstants.MaxLegthCodeSort) ARegionLine.RegionCode = ARegionLine.RegionCode.Substring(0, KROConstants.MaxLegthCodeSort);
                if (ARegionLine.RegionSort.Length > KROConstants.MaxLegthCodeSort) ARegionLine.RegionSort = ARegionLine.RegionSort.Substring(0, KROConstants.MaxLegthCodeSort);
                //конец подготовки данных
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateRegionCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_REGION_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ARegionLine.RegionID;
                        updateRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@RegionName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ARegionLine.RegionName;
                        updateRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@RegionCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ARegionLine.RegionCode;
                        updateRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@RegionSort", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ARegionLine.RegionSort;
                        updateRegionCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateRegionCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        # endregion
        # region UserBalanceCheckList
        public static List<UserBalanceCheckLine> GetUserBalanceCheckList(int AUserID)
        {
            List<UserBalanceCheckLine> lUserBalanceCheckList = new List<UserBalanceCheckLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand lSelectUserBalanceCheck = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_USER_BALANCE_CHECK_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    lSelectUserBalanceCheck.Parameters.AddWithValue("@UserID", AUserID);

                    connection.Open();
                    using (SqlDataReader UserBalanceCheckReader = lSelectUserBalanceCheck.ExecuteReader())
                    {
                        while (UserBalanceCheckReader.Read())
                        {
                            UserBalanceCheckLine lUserBalanceCheck = new UserBalanceCheckLine(Convert.ToInt32(UserBalanceCheckReader.GetValue(0)),
                                                                                              AUserID,
                                                                                              UserBalanceCheckReader["UserBalanceCheckID"] == DBNull.Value ? null : (int?)int.Parse(UserBalanceCheckReader["UserBalanceCheckID"].ToString()),
                                                                                              UserBalanceCheckReader.GetValue(1).ToString(),
                                                                                              UserBalanceCheckReader.GetValue(2).ToString(),
                                                                                              UserBalanceCheckReader.GetValue(3).ToString(),
                                                                                              UserBalanceCheckReader["Flag"] == DBNull.Value ? 0 : (long)long.Parse(UserBalanceCheckReader["Flag"].ToString()));
                            lUserBalanceCheckList.Add(lUserBalanceCheck);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lUserBalanceCheckList;
        }
        public static bool InsertUserBalanceCheck(UserBalanceCheckLine AUserBalanceCheckLine)
        {
            bool lResult = false;
            if ((AUserBalanceCheckLine.UserID > 0) &&
                (AUserBalanceCheckLine.RegionID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertUserBalanceCheckCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_USER_BALANCE_CHECK_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserBalanceCheckLine.UserID;
                        insertUserBalanceCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@BalanceRegionID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserBalanceCheckLine.RegionID;
                        insertUserBalanceCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserBalanceCheckLine.Flag;
                        insertUserBalanceCheckCommand.Parameters.Add(lParameter);

                        connection.Open();
                        AUserBalanceCheckLine.BalanceRegionID = (int)insertUserBalanceCheckCommand.ExecuteScalar();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }               
            }
            return lResult;
        }
        public static bool UpdateUserBalanceCheck(UserBalanceCheckLine AUserBalanceCheckLine)
        {
            bool lResult = false;
            if ((AUserBalanceCheckLine.BalanceRegionID > 0) &&
                (AUserBalanceCheckLine.UserID > 0) &&
                (AUserBalanceCheckLine.RegionID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateUserBalanceCheckCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_USER_BALANCE_CHECK_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserBalanceCheckLine.BalanceRegionID;
                        updateUserBalanceCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserBalanceCheckLine.UserID;
                        updateUserBalanceCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@BalanceRegionID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserBalanceCheckLine.RegionID;
                        updateUserBalanceCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserBalanceCheckLine.Flag;
                        updateUserBalanceCheckCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateUserBalanceCheckCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        # endregion
        # region ArtistCheckList
        public static List<UserArtistCheckLine> GetUserArtistCheckList(int AUserID)
        {
            List<UserArtistCheckLine> lUserArtistCheckList = new List<UserArtistCheckLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand lSelectUserArtistCheck = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_USER_ARTIST_CHECK_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    lSelectUserArtistCheck.Parameters.AddWithValue("@UserID", AUserID);

                    connection.Open();
                    using (SqlDataReader UserArtistCheckReader = lSelectUserArtistCheck.ExecuteReader())
                    {
                        while (UserArtistCheckReader.Read())
                        {
                            UserArtistCheckLine lUserArtistCheck = new UserArtistCheckLine(Convert.ToInt32(UserArtistCheckReader.GetValue(0)),
                                                                                            AUserID,
                                                                                            UserArtistCheckReader["UserArtistCheckID"] == DBNull.Value ? null : (int?)int.Parse(UserArtistCheckReader["UserArtistCheckID"].ToString()),
                                                                                            UserArtistCheckReader.GetValue(1).ToString(),
                                                                                            UserArtistCheckReader.GetValue(2).ToString(),
                                                                                            UserArtistCheckReader.GetValue(3).ToString(),
                                                                                            UserArtistCheckReader["Flag"] == DBNull.Value ? 0 : (long)long.Parse(UserArtistCheckReader["Flag"].ToString()));
                            lUserArtistCheckList.Add(lUserArtistCheck);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lUserArtistCheckList;
        }
        public static bool InsertUserArtistCheck(UserArtistCheckLine AUserArtistCheckLine)
        {
            bool lResult = false;
            if ((AUserArtistCheckLine.UserID > 0) &&
                (AUserArtistCheckLine.RegionID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertUserArtistCheckCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_USER_ARTIST_CHECK_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserArtistCheckLine.UserID;
                        insertUserArtistCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@ArtistRegionID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserArtistCheckLine.RegionID;
                        insertUserArtistCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserArtistCheckLine.Flag;
                        insertUserArtistCheckCommand.Parameters.Add(lParameter);

                        connection.Open();
                        AUserArtistCheckLine.ArtistRegionID = (int)insertUserArtistCheckCommand.ExecuteScalar();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateUserArtistCheck(UserArtistCheckLine AUserArtistCheckLine)
        {
            bool lResult = false;
            if ((AUserArtistCheckLine.ArtistRegionID > 0) &&
                (AUserArtistCheckLine.UserID > 0) &&
                (AUserArtistCheckLine.RegionID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateUserArtistCheckCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_USER_ARTIST_CHECK_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserArtistCheckLine.ArtistRegionID;
                        updateUserArtistCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserArtistCheckLine.UserID;
                        updateUserArtistCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@ArtistRegionID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserArtistCheckLine.RegionID;
                        updateUserArtistCheckCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AUserArtistCheckLine.Flag;
                        updateUserArtistCheckCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateUserArtistCheckCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        # endregion
        # region ReportsParameter
        public static List<ReportsParameterLine> GetReportsParameterList()
        {
            List<ReportsParameterLine> lReportsParameterList = new List<ReportsParameterLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand lSelectReportsParameter = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_REPORTS_PARAMETER_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    connection.Open();
                    using (SqlDataReader ReportsParameterReader = lSelectReportsParameter.ExecuteReader())
                    {


                        while (ReportsParameterReader.Read())
                        {
                            ReportsParameterLine lReportsParameter = new ReportsParameterLine(Convert.ToInt32(ReportsParameterReader.GetValue(0)),
                                                                                            ReportsParameterReader["MonthID"] == DBNull.Value ? -1 : (int) int.Parse(ReportsParameterReader["MonthID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(2).ToString(),
                                                                                            ReportsParameterReader["YearID"] == DBNull.Value ? -1 : (int) int.Parse(ReportsParameterReader["YearID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(4).ToString(),
                                                                                            ReportsParameterReader["TypeID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["TypeID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(6).ToString(),
                                                                                            ReportsParameterReader.GetValue(7).ToString(),
                                                                                            ReportsParameterReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["DepartmentID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(9).ToString(),
                                                                                            ReportsParameterReader.GetValue(10).ToString(),
                                                                                            ReportsParameterReader.GetValue(11).ToString(),
                                                                                            ReportsParameterReader.GetValue(12).ToString(),
                                                                                            ReportsParameterReader["ContractorID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["ContractorID"].ToString()),
                                                                                            ReportsParameterReader["ReportFlag"] == DBNull.Value ? 0 : (long)long.Parse(ReportsParameterReader["ReportFlag"].ToString()),
                                                                                            ReportsParameterReader["Comment"] == DBNull.Value ? "" : ReportsParameterReader["Comment"].ToString());
                            lReportsParameterList.Add(lReportsParameter);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }

            return lReportsParameterList;
        }
        public static List<ReportsParameterLine> GetReportsParameterListByUserIDAndBlock(int AUserID, bool AWithoutEconomicBlock, bool AWithoutFilialBlock, bool AWithoutRegionBlock)
        {
            List<ReportsParameterLine> lReportsParameterList = new List<ReportsParameterLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand lSelectReportsParameter = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_REPORTS_PARAMETER_BYUSERIDANDBLOCK_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = AUserID;
                    lSelectReportsParameter.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@WithoutEconomicBlock", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    if (AWithoutEconomicBlock) lParameter.Value = 1;
                    else lParameter.Value = 0;
                    lSelectReportsParameter.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@WithoutFilialBlock", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    if (AWithoutFilialBlock) lParameter.Value = 1;
                    else lParameter.Value = 0;
                    lSelectReportsParameter.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@WithoutRegionBlock", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    if (AWithoutRegionBlock) lParameter.Value = 1;
                    else lParameter.Value = 0;
                    lSelectReportsParameter.Parameters.Add(lParameter);

                    connection.Open();
                    using (SqlDataReader ReportsParameterReader = lSelectReportsParameter.ExecuteReader())
                    {


                        while (ReportsParameterReader.Read())
                        {
                            ReportsParameterLine lReportsParameter = new ReportsParameterLine(Convert.ToInt32(ReportsParameterReader.GetValue(0)),
                                                                                            ReportsParameterReader["MonthID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["MonthID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(2).ToString(),
                                                                                            ReportsParameterReader["YearID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["YearID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(4).ToString(),
                                                                                            ReportsParameterReader["TypeID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["TypeID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(6).ToString(),
                                                                                            ReportsParameterReader.GetValue(7).ToString(),
                                                                                            ReportsParameterReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["DepartmentID"].ToString()),
                                                                                            ReportsParameterReader.GetValue(9).ToString(),
                                                                                            ReportsParameterReader.GetValue(10).ToString(),
                                                                                            ReportsParameterReader.GetValue(11).ToString(),
                                                                                            ReportsParameterReader.GetValue(12).ToString(),
                                                                                            ReportsParameterReader["ContractorID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["ContractorID"].ToString()),
                                                                                            ReportsParameterReader["ReportFlag"] == DBNull.Value ? 0 : (long)long.Parse(ReportsParameterReader["ReportFlag"].ToString()),
                                                                                            ReportsParameterReader["Comment"] == DBNull.Value ? "" : ReportsParameterReader["Comment"].ToString());
                            lReportsParameterList.Add(lReportsParameter);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }

            return lReportsParameterList;
        }
        public static ReportsParameterLine GetOneReportParameterByID(int AReportParameterID)
        {
            ReportsParameterLine lReportParameterLine = new ReportsParameterLine();
            if (AReportParameterID > 0)
            {
                using (
                    SqlConnection connection = new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand lSelectReportParameterLine = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ONE_REPORTS_PARAMETER_BY_ID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ReportsParameterID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = AReportParameterID;
                        lSelectReportParameterLine.Parameters.Add(lParameter);
                        connection.Open();
                        using (
                            SqlDataReader ReportsParameterReader = lSelectReportParameterLine.ExecuteReader())
                        {


                            if (ReportsParameterReader.Read())
                            {
                                lReportParameterLine =
                                   new ReportsParameterLine(Convert.ToInt32(ReportsParameterReader.GetValue(0)),
                                                                            ReportsParameterReader["MonthID"] == DBNull.Value ? -1 : (int) int.Parse(ReportsParameterReader["MonthID"].ToString()),
                                                                            ReportsParameterReader.GetValue(2).ToString(),
                                                                            ReportsParameterReader["YearID"] == DBNull.Value ? -1 : (int) int.Parse(ReportsParameterReader["YearID"].ToString()),
                                                                            ReportsParameterReader.GetValue(4).ToString(),
                                                                            ReportsParameterReader["TypeID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["TypeID"].ToString()),
                                                                            ReportsParameterReader.GetValue(6).ToString(),
                                                                            ReportsParameterReader.GetValue(7).ToString(),
                                                                            ReportsParameterReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["DepartmentID"].ToString()),
                                                                            ReportsParameterReader.GetValue(9).ToString(),
                                                                            ReportsParameterReader.GetValue(10).ToString(),
                                                                            ReportsParameterReader.GetValue(11).ToString(),
                                                                            ReportsParameterReader.GetValue(12).ToString(),
                                                                            ReportsParameterReader["ContractorID"] == DBNull.Value ? -1 : (int)int.Parse(ReportsParameterReader["ContractorID"].ToString()),
                                                                            ReportsParameterReader["ReportFlag"] == DBNull.Value ? 0 : (long)long.Parse(ReportsParameterReader["ReportFlag"].ToString()),
                                                                            ReportsParameterReader["Comment"] == DBNull.Value ? "" : ReportsParameterReader["Comment"].ToString());
                            }
                        }
                    }
                    finally
                    {
                        if (connection != null) connection.Close();
                    }
                }
            }
            return lReportParameterLine;
        }
        public static List<BlockReportParameterHistoryLine> GetBlockReportParameterHistoryList(int AReportParameterID)
        {
            List<BlockReportParameterHistoryLine> lBlockReportParameterHistoryList = new List<BlockReportParameterHistoryLine>();
            if (AReportParameterID > 0)
            {
                using (
                    SqlConnection connection = new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand lSelectBlockReportParameterHistory = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_BLOCKREPORTPARAMETERHISTORY_BY_REPORTPARAMETERID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ReportParameterID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = AReportParameterID;
                        lSelectBlockReportParameterHistory.Parameters.Add(lParameter);
                        connection.Open();
                        using (
                            SqlDataReader BlockReportParameterHistoryReader = lSelectBlockReportParameterHistory.ExecuteReader())
                        {


                            while (BlockReportParameterHistoryReader.Read())
                            {
                                BlockReportParameterHistoryLine lBlockReportParameterHistory =
                                    new BlockReportParameterHistoryLine(
                                        Convert.ToInt32(BlockReportParameterHistoryReader.GetValue(0)),
                                        BlockReportParameterHistoryReader["ReportParameterID"] == DBNull.Value ? -1 : (int)int.Parse(BlockReportParameterHistoryReader["ReportParameterID"].ToString()),
                                        BlockReportParameterHistoryReader["UserID"] == DBNull.Value ? -1 : (int)int.Parse(BlockReportParameterHistoryReader["UserID"].ToString()),
                                        BlockReportParameterHistoryReader["UserFullName"].ToString(),
                                        BlockReportParameterHistoryReader["UserRoleID"] == DBNull.Value ? -1 : (int)int.Parse(BlockReportParameterHistoryReader["UserRoleID"].ToString()),
                                        BlockReportParameterHistoryReader["RoleName"].ToString(),
                                        BlockReportParameterHistoryReader["UserDepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(BlockReportParameterHistoryReader["UserDepartmentID"].ToString()),
                                        BlockReportParameterHistoryReader["DepartmentName"].ToString(),
                                        BlockReportParameterHistoryReader["OfficeName"].ToString(),
                                        BlockReportParameterHistoryReader["BlockValue"] == DBNull.Value ? 0 : (long)long.Parse(BlockReportParameterHistoryReader["BlockValue"].ToString()),
                                        BlockReportParameterHistoryReader["OperationDesription"].ToString(),
                                        BlockReportParameterHistoryReader["OperationData"] == DBNull.Value ? "" : BlockReportParameterHistoryReader["OperationData"].ToString(),
                                        BlockReportParameterHistoryReader["Comment"].ToString());
                                lBlockReportParameterHistoryList.Add(lBlockReportParameterHistory);
                            }
                        }
                    }
                    finally
                    {
                        if (connection != null) connection.Close();
                    }
                }
            }
            return lBlockReportParameterHistoryList;
        }
        public static bool InsertReportsParameter(ReportsParameterLine AReportsParameterLine)
        {
            bool lResult = false;
            if ((AReportsParameterLine.MonthID > 0) &&
                (AReportsParameterLine.YearID > 0) &&
                (AReportsParameterLine.TypeID > 0) &&
                (AReportsParameterLine.DepartmentID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertReportsParameterCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_REPORTS_PARAMETER_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@MonthID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.MonthID;
                        insertReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.YearID;
                        insertReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@TypeID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.TypeID;
                        insertReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.DepartmentID;
                        insertReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@ContractorID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = 1;
                        insertReportsParameterCommand.Parameters.Add(lParameter);
                    
                        lParameter = new SqlParameter("@ReportFlag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.ReportFlag;
                        insertReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Comment", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        if (AReportsParameterLine.Comment.Length == 0) lParameter.Value = DBNull.Value;
                        else lParameter.Value = AReportsParameterLine.Comment;
                        insertReportsParameterCommand.Parameters.Add(lParameter);

                        connection.Open();
                        AReportsParameterLine.SetReportParameterID((int)insertReportsParameterCommand.ExecuteScalar());
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateReportsParameter(ReportsParameterLine AReportsParameterLine)
        {
            bool lResult = false;
            if ((AReportsParameterLine.ReportParameterID > 0) && 
                (AReportsParameterLine.MonthID > 0) &&
                (AReportsParameterLine.YearID > 0) &&
                (AReportsParameterLine.TypeID > 0) &&
                (AReportsParameterLine.DepartmentID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateReportsParameterCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_REPORTS_PARAMETER_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.ReportParameterID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MonthID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.MonthID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.YearID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@TypeID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.TypeID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.DepartmentID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@ContractorID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = 1;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@ReportFlag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.ReportFlag;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Comment", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        if (AReportsParameterLine.Comment.Length == 0) lParameter.Value = DBNull.Value;
                        else lParameter.Value = AReportsParameterLine.Comment;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateReportsParameterCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateOnlyFlagReportsParameter(HttpSessionState ASession,  ReportsParameterLine AReportsParameterLine, string AOperationDesription, string AComment = "")
        {
            var lUserID = AuthenticationManager.GetAuthenticationUserModel(ASession).ID;
            bool lResult = false;
            if (AReportsParameterLine.ReportParameterID > 0)
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateReportsParameterCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_ONLY_FLAG_REPORTS_PARAMETER_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.ReportParameterID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@ReportFlag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.ReportFlag;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = lUserID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@OperationDesription", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        if (AOperationDesription.Length == 0) lParameter.Value = DBNull.Value;
                        else lParameter.Value = AOperationDesription;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Comment", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        if (AComment.Length == 0) lParameter.Value = DBNull.Value;
                        else lParameter.Value = AComment;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateReportsParameterCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            //сохраним системное сообщение
            if (lResult) ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgBlocReportParameter + " | ID = " + AReportsParameterLine.ReportParameterID.ToString() + " | Flag = " + AReportsParameterLine.ReportFlag.ToString(), 3), KROFlag.flLogReportParameter, lUserID);
            else ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgBlocReportParameter + " | " + " | ID = " + AReportsParameterLine.ReportParameterID.ToString() + " | Flag = " + AReportsParameterLine.ReportFlag.ToString(), 2), KROFlag.flLogReportParameter, lUserID);
            return lResult;
        }
        public static bool UpdateReportsParameter(HttpSessionState ASession, int AReportParameterID, int AReportFlag, string AOperationDesription, string AComment = "")
        {
            var lUserID = AuthenticationManager.GetAuthenticationUserModel(ASession).ID;
            bool lResult = false;
            if (AReportParameterID > 0)
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateReportsParameterCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_ONLY_FLAG_REPORTS_PARAMETER_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportParameterID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@ReportFlag", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportFlag;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = lUserID;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@OperationDesription", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        if (AOperationDesription.Length == 0) lParameter.Value = DBNull.Value;
                        else lParameter.Value = AOperationDesription;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Comment", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        if (AComment.Length == 0) lParameter.Value = DBNull.Value;
                        else lParameter.Value = AComment;
                        updateReportsParameterCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateReportsParameterCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            //сохраним системное сообщение
            if (lResult) ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgBlocReportParameter + " | ID = " + AReportParameterID.ToString() + " | Flag = " + AReportFlag.ToString(), 3), KROFlag.flLogReportParameter, lUserID);
            else ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgBlocReportParameter + " | " + " | ID = " + AReportParameterID.ToString() + " | Flag = " + AReportFlag.ToString(), 2), KROFlag.flLogReportParameter, lUserID);
            return lResult;
        }
        public static void AddChangeReportParameterLineInSession(HttpSessionState ASession, ReportsParameterLine AReportParameterLine)
        {
            ASession[KROConstants.SESSION_CHANGE_REPORTPARAMETER_LINE] = AReportParameterLine;
        }
        public static void AddChangeReportParameterLineBlockInSession(HttpSessionState ASession, ReportsParameterLine AReportParameterLine)
        {
            ASession[KROConstants.SESSION_CHANGE_REPORTPARAMETER_LINE_BLOCK] = AReportParameterLine;
        }
        public static ReportsParameterLine GetChangeReportParameterLineFromSession(HttpSessionState ASession)
        {
            ReportsParameterLine lReportParameterLine = new ReportsParameterLine();
            if ((ASession != null) && (ASession.Count > 0))
            {
                try
                {   //попробуем получить изменяемый список показателей в периоде
                    lReportParameterLine = (ReportsParameterLine)ASession[KROConstants.SESSION_CHANGE_REPORTPARAMETER_LINE];
                }
                catch { }
            }
            return lReportParameterLine;
        }
        public static ReportsParameterLine GetChangeReportParameterLineBlockFromSession(HttpSessionState ASession)
        {
            ReportsParameterLine lReportParameterLine = new ReportsParameterLine();
            if ((ASession != null) && (ASession.Count > 0))
            {
                try
                {   //попробуем получить изменяемый список показателей в периоде
                    lReportParameterLine = (ReportsParameterLine)ASession[KROConstants.SESSION_CHANGE_REPORTPARAMETER_LINE_BLOCK];
                }
                catch { }
            }
            return lReportParameterLine;
        }
        # endregion
        # region MounthList
        public static List<MounthLine> GetMounthList()
        {
            List<MounthLine> lMounthList = new List<MounthLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllMounthCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SelectAllMounthsSQL,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader MounthReader = selectAllMounthCommand.ExecuteReader())
                    {
                        while (MounthReader.Read())
                        {
                            MounthLine currentRole = new MounthLine(Convert.ToInt32(MounthReader.GetValue(0)),
                                                                    MounthReader.GetValue(1).ToString());
                            lMounthList.Add(currentRole);
                        }               
                    }
                }
                    finally { if (connection != null) connection.Close(); }
            }
            return lMounthList;
        }
        # endregion
        # region YearList
        public static List<YearLine> GetYearList(bool ADesc = true)
        {
            List<YearLine> lYearList = new List<YearLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {   string lSql = "";
                if (ADesc) lSql = KROSQLs.SelectAllYearsSQLDesc; else lSql = KROSQLs.SelectAllYearsSQL;
                    SqlCommand selectAllYearCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = lSql,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader YearReader = selectAllYearCommand.ExecuteReader())
                    {
                        while (YearReader.Read())
                        {
                            YearLine currentRole = new YearLine(Convert.ToInt32(YearReader.GetValue(0)),
                                                                    YearReader.GetValue(1).ToString());
                            lYearList.Add(currentRole);
                        }
                    }
                }
                    finally { if (connection != null) connection.Close(); }
            }
            return lYearList;
        }
        # endregion
        # region DepartmentList
        public static List<DepartmentLine> GetDepartmentList()
        {
            List<DepartmentLine> lDepartmentList = new List<DepartmentLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllDepartmentCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SelectAllDepartmentsSQL,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader DepartmentReader = selectAllDepartmentCommand.ExecuteReader())
                    {                    
                        while (DepartmentReader.Read())
                        {
                            DepartmentLine currentRole = new DepartmentLine(Convert.ToInt32(DepartmentReader.GetValue(0)),
                                                                            DepartmentReader.GetValue(1).ToString(),
                                                                            DepartmentReader.GetValue(2).ToString(),
                                                                            DepartmentReader.GetValue(3).ToString(),
                                                                            DepartmentReader["DepartmentCategory"] == DBNull.Value ? 0 : (Int32)Int32.Parse(DepartmentReader["DepartmentCategory"].ToString()));
                            lDepartmentList.Add(currentRole);
                        }
                    }
                }
                    finally { if (connection != null) connection.Close(); }
            }
            return lDepartmentList;
        }
        public static bool InsertDepartment(DepartmentLine ADepartmentLine)
        {
            bool lResult = false;

            if ((ADepartmentLine.DepartmentName.Length > 0) &&
                (ADepartmentLine.DepartmentCode.Length > 0) &&
                (ADepartmentLine.DepartmentSort.Length > 0))
            {
                //подготовка данных подрежем первые 10 символов у ADepartmentCode и ADepartmentSort и подправим ADepartmentCategory
                if (ADepartmentLine.DepartmentCode.Length > KROConstants.MaxLegthCodeSort) ADepartmentLine.DepartmentCode = ADepartmentLine.DepartmentCode.Substring(0, KROConstants.MaxLegthCodeSort);
                if (ADepartmentLine.DepartmentSort.Length > KROConstants.MaxLegthCodeSort) ADepartmentLine.DepartmentSort = ADepartmentLine.DepartmentSort.Substring(0, KROConstants.MaxLegthCodeSort);
                if (ADepartmentLine.DepartmentCategory < 0) ADepartmentLine.SetFlag(0);
                //конец подготовки данных
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertDepartmentCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_DEPARTMENT_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@DepartmentName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepartmentLine.DepartmentName;
                        insertDepartmentCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepartmentLine.DepartmentCode;
                        insertDepartmentCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentSort", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepartmentLine.DepartmentSort;
                        insertDepartmentCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentCategory", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ADepartmentLine.DepartmentCategory;
                        insertDepartmentCommand.Parameters.Add(lParameter);

                        connection.Open();
                        ADepartmentLine.SetDepartmentID((int)insertDepartmentCommand.ExecuteScalar());
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateDepartmentCheck(DepartmentLine ADepartmentLine)
        {
            bool lResult = false;
            if ((ADepartmentLine.DepartmentID > 0) &&
                (ADepartmentLine.DepartmentName.Length > 0) &&
                (ADepartmentLine.DepartmentCode.Length > 0) &&
                (ADepartmentLine.DepartmentSort.Length > 0))
            {
                //подготовка данных подрежем первые 10 символов у ADepartmentCode и ADepartmentSort и подправим ADepartmentCategory
                if (ADepartmentLine.DepartmentCode.Length > KROConstants.MaxLegthCodeSort) ADepartmentLine.DepartmentCode = ADepartmentLine.DepartmentCode.Substring(0, KROConstants.MaxLegthCodeSort);
                if (ADepartmentLine.DepartmentSort.Length > KROConstants.MaxLegthCodeSort) ADepartmentLine.DepartmentSort = ADepartmentLine.DepartmentSort.Substring(0, KROConstants.MaxLegthCodeSort);
                if (ADepartmentLine.DepartmentCategory < 0) ADepartmentLine.SetFlag(0);
                //конец подготовки данных
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateDepartmentCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_DEPARTMENT_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepartmentLine.DepartmentID;
                        updateDepartmentCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepartmentLine.DepartmentName;
                        updateDepartmentCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepartmentLine.DepartmentCode;
                        updateDepartmentCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentSort", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepartmentLine.DepartmentSort;
                        updateDepartmentCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentCategory", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ADepartmentLine.DepartmentCategory;
                        updateDepartmentCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateDepartmentCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static List<DepartmentLine> GetDepartmentListForRKCUReport(int AUserID, int AMonthID, int AYearID)
        {
            List<DepartmentLine> lDepartmentList = new List<DepartmentLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllDepartmentCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.GET_DEPARTMENTLIST_BY_USERID_MONTHID_YEARID_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AUserID;
                    selectAllDepartmentCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@MonthID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AMonthID;
                    selectAllDepartmentCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@YearID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AYearID;
                    selectAllDepartmentCommand.Parameters.Add(lParameter);

                    connection.Open();
                    using (SqlDataReader DepartmentReader = selectAllDepartmentCommand.ExecuteReader())
                    {
                        int lDepartmentID;
                        while (DepartmentReader.Read())
                        {
                            lDepartmentID = -1;
                            lDepartmentID = int.Parse(DepartmentReader["DepartmentID"].ToString());
                            if ((lDepartmentID>0) && 
                                (lDepartmentList.Find(Line => Line.DepartmentID == lDepartmentID) == null))
                            {
                                DepartmentLine currentRole = new DepartmentLine(lDepartmentID,
                                                                                DepartmentReader["DepartmentName"].ToString(),
                                                                                DepartmentReader["DepartmentSort"].ToString(),
                                                                                DepartmentReader["DepartmentCode"].ToString(),
                                                                                0);
                                lDepartmentList.Add(currentRole);
                            }
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lDepartmentList;
        }
        # endregion
        # region DepIncludeRegionList
        public static List<DepIncludeRegionLine> GetDepIncludeRegionList()
        {
            List<DepIncludeRegionLine> lDepIncludeRegionList = new List<DepIncludeRegionLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllDepIncludeRegionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_DEPARTMENT_PERIOD_INCLUDE_REGION_SP_NAME,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader DepIncludeRegionReader = selectAllDepIncludeRegionCommand.ExecuteReader())
                    {
                        while (DepIncludeRegionReader.Read())
                        {
                            DepIncludeRegionLine currentRole = new DepIncludeRegionLine(int.Parse(DepIncludeRegionReader["ID"].ToString()),
                                                                          DepIncludeRegionReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["DepartmentID"].ToString()),
                                                                          DepIncludeRegionReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentName"],
                                                                          DepIncludeRegionReader["DepartmentCode"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentCode"],
                                                                          DepIncludeRegionReader["DepartmentSort"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentSort"],
                                                                          DepIncludeRegionReader["MonthIN"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["MonthIN"].ToString()),
                                                                          DepIncludeRegionReader["MonthName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["MonthName"],
                                                                          DepIncludeRegionReader["YearIN"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["YearIN"].ToString()),
                                                                          DepIncludeRegionReader["YearName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["YearName"].ToString(),
                                                                          DepIncludeRegionReader["DateIn"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(DepIncludeRegionReader["DateIn"].ToString()),
                                                                          DepIncludeRegionReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(DepIncludeRegionReader["Flag"].ToString())
                                                                                  );
                            lDepIncludeRegionList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lDepIncludeRegionList;
        }

        public static List<DepIncludeRegionLine> GetDepIncludeRegionListByDepartmentID(int ADepartmentID)
        {
            List<DepIncludeRegionLine> lDepIncludeRegionList = new List<DepIncludeRegionLine>();
            if (ADepartmentID > 0)
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllDepIncludeRegionCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_DEPARTMENT_PERIOD_INCLUDE_REGION_BY_DEPARTMENT_ID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ADepartmentID;
                        selectAllDepIncludeRegionCommand.Parameters.Add(lParameter);

                        connection.Open();
                        using (SqlDataReader DepIncludeRegionReader = selectAllDepIncludeRegionCommand.ExecuteReader())
                        {
                            while (DepIncludeRegionReader.Read())
                            {
                                DepIncludeRegionLine currentRole = new DepIncludeRegionLine(int.Parse(DepIncludeRegionReader["ID"].ToString()),
                                                                              DepIncludeRegionReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["DepartmentID"].ToString()),
                                                                              DepIncludeRegionReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentName"],
                                                                              DepIncludeRegionReader["DepartmentCode"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentCode"],
                                                                              DepIncludeRegionReader["DepartmentSort"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentSort"],
                                                                              DepIncludeRegionReader["MonthIN"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["MonthIN"].ToString()),
                                                                              DepIncludeRegionReader["MonthName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["MonthName"],
                                                                              DepIncludeRegionReader["YearIN"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["YearIN"].ToString()),
                                                                              DepIncludeRegionReader["YearName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["YearName"].ToString(),
                                                                              DepIncludeRegionReader["DateIn"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(DepIncludeRegionReader["DateIn"].ToString()),
                                                                              DepIncludeRegionReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(DepIncludeRegionReader["Flag"].ToString())
                                                                                      );
                                lDepIncludeRegionList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lDepIncludeRegionList;
        }
        public static List<DepIncludeRegionLine> GetDepIncludeRegionByActualDate(DateTime AActualDate)
        {
            List<DepIncludeRegionLine>  lDepIncludeRegionList = new List<DepIncludeRegionLine>();
            if (AActualDate != null)
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllDepListInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_DEPARTMENT_PERIOD_INCLUDE_REGION_BY_ACTUALDATE_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        selectAllDepListInPeriodCommand.Parameters.AddWithValue("@ActualDate", AActualDate.ToString(KROConstants.DateCultureSQL));

                        connection.Open();
                        using (SqlDataReader DepIncludeRegionReader = selectAllDepListInPeriodCommand.ExecuteReader())
                        {
                            while (DepIncludeRegionReader.Read())
                            {
                                DepIncludeRegionLine currentRole = new DepIncludeRegionLine(int.Parse(DepIncludeRegionReader["ID"].ToString()),
                                                                              DepIncludeRegionReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["DepartmentID"].ToString()),
                                                                              DepIncludeRegionReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentName"],
                                                                              DepIncludeRegionReader["DepartmentCode"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentCode"],
                                                                              DepIncludeRegionReader["DepartmentSort"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["DepartmentSort"],
                                                                              DepIncludeRegionReader["MonthIN"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["MonthIN"].ToString()),
                                                                              DepIncludeRegionReader["MonthName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["MonthName"],
                                                                              DepIncludeRegionReader["YearIN"] == DBNull.Value ? -1 : (int)int.Parse(DepIncludeRegionReader["YearIN"].ToString()),
                                                                              DepIncludeRegionReader["YearName"] == DBNull.Value ? string.Empty : (string)DepIncludeRegionReader["YearName"].ToString(),
                                                                              DepIncludeRegionReader["DateIn"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(DepIncludeRegionReader["DateIn"].ToString()),
                                                                              DepIncludeRegionReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(DepIncludeRegionReader["Flag"].ToString())
                                                                                       );
                                lDepIncludeRegionList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lDepIncludeRegionList;
        }
        public static bool InsertDepIncludeRegion(DepIncludeRegionLine ADepIncludeRegionLine, List<RegionDirectionInPeriodLine> ARegionDirectionInPeriodList)
        {
            bool lResult = false;

            if ((ADepIncludeRegionLine.DepartmentID> 0) &&
                (ADepIncludeRegionLine.MounthID > 0) &&
                (ADepIncludeRegionLine.YearID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertDepIncludeRegionCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_DEPARTMENT_PERIOD_INCLUDE_REGION_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepIncludeRegionLine.DepartmentID;
                        insertDepIncludeRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MonthIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepIncludeRegionLine.MounthID;
                        insertDepIncludeRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepIncludeRegionLine.YearID;
                        insertDepIncludeRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ADepIncludeRegionLine.Flag;
                        insertDepIncludeRegionCommand.Parameters.Add(lParameter);

                        connection.Open();
                        SqlTransaction lTransaction = connection.BeginTransaction();
                        try
                        {
                            insertDepIncludeRegionCommand.Transaction = lTransaction;
                            ADepIncludeRegionLine.SetID((int)insertDepIncludeRegionCommand.ExecuteScalar());
                           // пошли сохранять остальные параметры
                            foreach (RegionDirectionInPeriodLine lRegionDirectionLine in ARegionDirectionInPeriodList)
                            {
                                lRegionDirectionLine.DepPerInclRegID = ADepIncludeRegionLine.ID;
                                if (lRegionDirectionLine.ID > 0) UpdateRegionDirectionCheck(lRegionDirectionLine, connection, lTransaction);
                                else
                                {
                                    try {
                                       InsertRegionDirection(lRegionDirectionLine, connection, lTransaction);
                                    }
                                    catch (Exception expinsert)
                                    {
                                        lRegionDirectionLine.SetID(-1);
                                        throw expinsert;
                                    }
                                }
                            }
                            lTransaction.Commit();
                            connection.Close();
                            lResult = true;
                        }
                        catch
                        {
                            lTransaction.Rollback();
                            throw;
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateDepIncludeRegionCheck(DepIncludeRegionLine ADepIncludeRegionLine, List<RegionDirectionInPeriodLine> ARegionDirectionInPeriodList)
        {
            bool lResult = false;
            if ((ADepIncludeRegionLine.ID > 0) &&
                (ADepIncludeRegionLine.DepartmentID > 0) &&
                (ADepIncludeRegionLine.MounthID > 0) &&
                (ADepIncludeRegionLine.YearID > 0) &&
                (ARegionDirectionInPeriodList !=null))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateDepIncludeRegionCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_DEPARTMENT_PERIOD_INCLUDE_REGION_BY_ID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepIncludeRegionLine.ID;
                        updateDepIncludeRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepIncludeRegionLine.DepartmentID;
                        updateDepIncludeRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MonthIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepIncludeRegionLine.MounthID;
                        updateDepIncludeRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepIncludeRegionLine.YearID;
                        updateDepIncludeRegionCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ADepIncludeRegionLine.Flag;
                        updateDepIncludeRegionCommand.Parameters.Add(lParameter);

                        connection.Open();
                        SqlTransaction lTransaction = connection.BeginTransaction();
                        try
                        {
                            updateDepIncludeRegionCommand.Transaction = lTransaction;
                            updateDepIncludeRegionCommand.ExecuteNonQuery();
                            // пошли сохранять остальные параметры
                            foreach (RegionDirectionInPeriodLine lRegionDirectionLine in ARegionDirectionInPeriodList)
                            {
                                lRegionDirectionLine.DepPerInclRegID = ADepIncludeRegionLine.ID;
                                if (lRegionDirectionLine.ID > 0) UpdateRegionDirectionCheck(lRegionDirectionLine, connection, lTransaction);
                                else
                                {
                                    try
                                    {
                                        InsertRegionDirection(lRegionDirectionLine, connection, lTransaction);
                                    }
                                    catch (Exception expinsert)
                                    {
                                        lRegionDirectionLine.SetID(-1);
                                        throw expinsert;
                                    }
                                }
                            }
                            lTransaction.Commit();
                            connection.Close();
                            lResult = true;
                        }
                        catch 
                        {
                          lTransaction.Rollback();
                          throw;
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        # endregion
        # region DepListInPeriodList
        public static List<DepListInPeriodLine> GetDepListInPeriodList()
        {
            List<DepListInPeriodLine> lDepListInPeriodList = new List<DepListInPeriodLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllDepListInPeriodCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_DEPLISTINPERIOD_SP_NAME,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader DepListInPeriodReader = selectAllDepListInPeriodCommand.ExecuteReader())
                    {
                        while (DepListInPeriodReader.Read())
                        {
                            DepListInPeriodLine currentRole = new DepListInPeriodLine(int.Parse(DepListInPeriodReader["ID"].ToString()),
                                                                          DepListInPeriodReader["MounthID"] == DBNull.Value ? -1 : (int)int.Parse(DepListInPeriodReader["MounthID"].ToString()),
                                                                          DepListInPeriodReader["MonthName"] == DBNull.Value ? string.Empty : (string)DepListInPeriodReader["MonthName"],
                                                                          DepListInPeriodReader["YearID"] == DBNull.Value ? -1 : (int)int.Parse(DepListInPeriodReader["YearID"].ToString()),
                                                                          DepListInPeriodReader["YearName"] == DBNull.Value ? string.Empty : (string)DepListInPeriodReader["YearName"].ToString(),
                                                                          DepListInPeriodReader["DateIn"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(DepListInPeriodReader["DateIn"].ToString()),
                                                                          DepListInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(DepListInPeriodReader["Flag"].ToString())
                                                                                  );
                            lDepListInPeriodList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lDepListInPeriodList;
        }
        public static List<DepListInPeriodLine> GetDepListInPeriodListByActualDate(DateTime AActualDate)
        {
            List<DepListInPeriodLine> lDepListInPeriodList = new List<DepListInPeriodLine>();
            if (AActualDate != null)
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllDepListInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_DEPLISTINPERIOD_BY_ACTUALDATE_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        selectAllDepListInPeriodCommand.Parameters.AddWithValue("@ActualDate", AActualDate.ToString(KROConstants.DateCultureSQL));

                        connection.Open();
                        using (SqlDataReader DepListInPeriodReader = selectAllDepListInPeriodCommand.ExecuteReader())
                        {
                            while (DepListInPeriodReader.Read())
                            {
                                DepListInPeriodLine currentRole = new DepListInPeriodLine(int.Parse(DepListInPeriodReader["ID"].ToString()),
                                                                              DepListInPeriodReader["MounthID"] == DBNull.Value ? -1 : (int)int.Parse(DepListInPeriodReader["MounthID"].ToString()),
                                                                              DepListInPeriodReader["MonthName"] == DBNull.Value ? string.Empty : (string)DepListInPeriodReader["MonthName"],
                                                                              DepListInPeriodReader["YearID"] == DBNull.Value ? -1 : (int)int.Parse(DepListInPeriodReader["YearID"].ToString()),
                                                                              DepListInPeriodReader["YearName"] == DBNull.Value ? string.Empty : (string)DepListInPeriodReader["YearName"].ToString(),
                                                                              DepListInPeriodReader["DateIn"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(DepListInPeriodReader["DateIn"].ToString()),
                                                                              DepListInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(DepListInPeriodReader["Flag"].ToString())
                                                                                      );
                                lDepListInPeriodList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lDepListInPeriodList;
        }
        public static bool InsertDepListInPeriod(DepListInPeriodLine ADepListInPeriodLine, List<DepartmentIDInPeriodLine> ADepartmentIDInPeriodList)
        {
            bool lResult = false;

            if ((ADepListInPeriodLine.MounthID > 0) &&
                (ADepListInPeriodLine.YearID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertDepListInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_DEPLISTINPERIOD_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@MonthIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepListInPeriodLine.MounthID;
                        insertDepListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepListInPeriodLine.YearID;
                        insertDepListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ADepListInPeriodLine.Flag;
                        insertDepListInPeriodCommand.Parameters.Add(lParameter);

                        connection.Open();
                        SqlTransaction lTransaction = connection.BeginTransaction();
                        try
                        {
                            insertDepListInPeriodCommand.Transaction = lTransaction;
                            ADepListInPeriodLine.SetID((int)insertDepListInPeriodCommand.ExecuteScalar());
                            // пошли сохранять остальные параметры
                            foreach (DepartmentIDInPeriodLine lDepartmentIDInPeriodLine in ADepartmentIDInPeriodList)
                            {
                                lDepartmentIDInPeriodLine.DepListInPeriodID = ADepListInPeriodLine.ID;
                                if (lDepartmentIDInPeriodLine.ID > 0) UpdateDepartmentIDInPeriodCheck(lDepartmentIDInPeriodLine, connection, lTransaction);
                                else
                                {
                                    try {
                                      InsertDepartmentIDInPeriod(lDepartmentIDInPeriodLine, connection, lTransaction);
                                    }
                                    catch (Exception expinsert)
                                    {
                                        lDepartmentIDInPeriodLine.SetID(-1);
                                        throw expinsert;
                                    }
                                }
                            }
                            lTransaction.Commit();
                            connection.Close();
                            lResult = true;
                        }
                        catch
                        {
                            lTransaction.Rollback();
                            throw;
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateDepListInPeriodCheck(DepListInPeriodLine ADepListInPeriodLine, List<DepartmentIDInPeriodLine> ADepartmentIDInPeriodList)
        {
            bool lResult = false;
            if ((ADepListInPeriodLine.ID > 0) &&
                (ADepListInPeriodLine.MounthID > 0) &&
                (ADepListInPeriodLine.YearID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateDepListInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_DEPLISTINPERIOD_BY_ID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepListInPeriodLine.ID;
                        updateDepListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MonthIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepListInPeriodLine.MounthID;
                        updateDepListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = ADepListInPeriodLine.YearID;
                        updateDepListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ADepListInPeriodLine.Flag;
                        updateDepListInPeriodCommand.Parameters.Add(lParameter);

                        connection.Open();
                        SqlTransaction lTransaction = connection.BeginTransaction();
                        try
                        {
                            updateDepListInPeriodCommand.Transaction = lTransaction;
                            updateDepListInPeriodCommand.ExecuteNonQuery();
                            // пошли сохранять остальные параметры
                            foreach (DepartmentIDInPeriodLine lDepartmentIDInPeriodLine in ADepartmentIDInPeriodList)
                            {
                                lDepartmentIDInPeriodLine.DepListInPeriodID = ADepListInPeriodLine.ID;
                                if (lDepartmentIDInPeriodLine.ID > 0) UpdateDepartmentIDInPeriodCheck(lDepartmentIDInPeriodLine, connection, lTransaction);
                                else
                                {
                                    try { 
                                        InsertDepartmentIDInPeriod(lDepartmentIDInPeriodLine, connection, lTransaction);
                                    }
                                    catch (Exception expinsert)
                                    {
                                        lDepartmentIDInPeriodLine.SetID(-1);
                                        throw expinsert;
                                    }
                                }
                            }
                            lTransaction.Commit();
                            connection.Close();
                            lResult = true;
                        }
                        catch 
                        {
                            lTransaction.Rollback();
                            throw;
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        # endregion
        # region TypeList
        public static List<TypeLine> GetTypeList()
        {
            List<TypeLine> lTypeList = new List<TypeLine>();
            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllTypeCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SelectAllTypesSQL,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader TypeReader = selectAllTypeCommand.ExecuteReader())
                    {
                        while (TypeReader.Read())
                        {
                            TypeLine currentRole = new TypeLine(Convert.ToInt32(TypeReader.GetValue(0)),
                                                                        TypeReader.GetValue(1).ToString(),
                                                                        TypeReader.GetValue(2).ToString());
                            lTypeList.Add(currentRole);
                        }
                    }
                }
                    finally { if (connection != null) connection.Close(); }
            }
            return lTypeList;
        }
        # endregion
        # region LogOutList
        public static List<LogOutLine> GetLogOutList(DateTime ADateIn, DateTime ADateOut)
        {
            List<LogOutLine> lLogOutList = new List<LogOutLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand lSelectAnyLogOutMessage = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ANY_LOG_MESSAGE_BY_DATE_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    
                    lSelectAnyLogOutMessage.Parameters.AddWithValue("@DateIn", ADateIn.ToString(KROConstants.DateCultureSQL));
                    lSelectAnyLogOutMessage.Parameters.AddWithValue("@DateOut", ADateOut.AddDays(1).ToString(KROConstants.DateCultureSQL)); 
                     
                    connection.Open();
                    using (SqlDataReader LogOutMessageReader = lSelectAnyLogOutMessage.ExecuteReader())
                    {
                        while (LogOutMessageReader.Read())
                        {
                            LogOutLine llLogOutLine = new LogOutLine(long.Parse(LogOutMessageReader["ID"].ToString()),
                                                                          LogOutMessageReader["LogDate"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(LogOutMessageReader["LogDate"].ToString()),
                                                                          LogOutMessageReader["UserID"] == DBNull.Value ? null : (int?) int.Parse(LogOutMessageReader["UserID"].ToString()),
                                                                          LogOutMessageReader["Family"] == DBNull.Value ? string.Empty : (string)LogOutMessageReader["Family"],
                                                                          LogOutMessageReader["FirstName"] == DBNull.Value ? string.Empty : (string)LogOutMessageReader["FirstName"],
                                                                          LogOutMessageReader["SecondName"] == DBNull.Value ? string.Empty : (string)LogOutMessageReader["SecondName"],
                                                                          LogOutMessageReader["UserFlag"] == DBNull.Value ? 0 : (long)long.Parse(LogOutMessageReader["UserFlag"].ToString()),
                                                                          LogOutMessageReader["RoleName"] == DBNull.Value ? string.Empty : (string)LogOutMessageReader["RoleName"],
                                                                          LogOutMessageReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)LogOutMessageReader["DepartmentName"],
                                                                          LogOutMessageReader["LogType"] == DBNull.Value ? 0 : (long)long.Parse(LogOutMessageReader["LogType"].ToString()),
                                                                          LogOutMessageReader["LogMessage"] == DBNull.Value ? string.Empty : (string)LogOutMessageReader["LogMessage"]
                                                                                  );
                            lLogOutList.Add(llLogOutLine);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lLogOutList;
        }
        public static bool InsertLogOut(string ALogMessage, long ALogType, int AUserID)
        {
            bool lResult = false;
            if (ALogMessage.Length > 0)
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertLogOutCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_LOGUOT_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@LogDate", SqlDbType.DateTime);
                        lParameter.IsNullable = false;
                        lParameter.Value = DateTime.Now;
                        insertLogOutCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@UserID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        if (AUserID<=0)  lParameter.Value = DBNull.Value;
                        else lParameter.Value = AUserID;
                        insertLogOutCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@LogType", SqlDbType.BigInt);
                        lParameter.IsNullable = false;
                        lParameter.Value = ALogType;
                        insertLogOutCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Message", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = ALogMessage;
                        insertLogOutCommand.Parameters.Add(lParameter);

                        connection.Open();
                        long lLogID = (long)insertLogOutCommand.ExecuteScalar();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool ClearLogOut()
        {
            bool lResult = false;
             using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand clearLogOutCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.CLEAR_LOGOUT_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        connection.Open();
                        clearLogOutCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            return lResult;
        }
        /// <summary>
        /// Генерирует системное сообщение
        /// </summary>
        /// <param name="ALogMessage">произвольный текст сообщения с указанием таблицы БД и места/процедуры возникновения</param>
        /// <param name="AMessageType">0-Unknown, 1=Message, 2=Error, 3=Update, 4=Insert, 
        ///                            5=Delete, 6=ChangeFlag ,7-Session, 8-OutSession 
        ///                            полный список параметров в классе KROConstants</param>
        /// <returns>Системное сообщение</returns>
        public static string GetLogMessage(string ALogMessage, int AMessageType)
        {
            string lLogMessage = "";
            if ((AMessageType < 0) || (AMessageType > KROConstants.lgTypeLogMessage.Length - 1))
            {
                lLogMessage = KROConstants.lgTypeLogMessage[0];
            }
            else lLogMessage = KROConstants.lgTypeLogMessage[AMessageType];
            return lLogMessage + " | " + ALogMessage;
        }
        # endregion
        #region RegionDirectionList
        public static List<RegionDirectionInPeriodLine> GetRegionDirectionListByDepPerInclID(int ADepPerInclID)
        {
            List<RegionDirectionInPeriodLine> lRegionDirectionList = new List<RegionDirectionInPeriodLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllRegionDirectionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_REGION_DIRECTION_IN_PERIOD_BY_DEPPERINCLID_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@DepartmentRegionID", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = ADepPerInclID;
                    selectAllRegionDirectionCommand.Parameters.Add(lParameter);
                    
                    connection.Open();
                    using (SqlDataReader RegionDirectionReader = selectAllRegionDirectionCommand.ExecuteReader())
                    {
                        while (RegionDirectionReader.Read())
                        {
                            RegionDirectionInPeriodLine currentRole = 
                            new  RegionDirectionInPeriodLine(int.Parse(RegionDirectionReader["ID"].ToString()),
                                                                       RegionDirectionReader["DepPerInclRegID"] == DBNull.Value ? -1 : (int)int.Parse(RegionDirectionReader["DepPerInclRegID"].ToString()),
                                                                       RegionDirectionReader["RegionID"] == DBNull.Value ? -1 : (int)int.Parse(RegionDirectionReader["RegionID"].ToString()),
                                                                       RegionDirectionReader["RegionName"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionName"],
                                                                       RegionDirectionReader["RegionSort"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionSort"],
                                                                       RegionDirectionReader["RegionCode"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionCode"],
                                                                       RegionDirectionReader["RegionDirectionName"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionDirectionName"].ToString(),
                                                                       RegionDirectionReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(RegionDirectionReader["Flag"].ToString())
                                                                                );
                            lRegionDirectionList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRegionDirectionList;
        }
        public static RegionDirectionInPeriodLine GetRegionDirectionListByDepPerInclIDAndRegionID(int ADepPerInclID,int ARegionID)
        {
            RegionDirectionInPeriodLine lRegionDirectionLine;

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllRegionDirectionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_DEPARTMENT_PERIOD_INCLUDE_REGION_BY_DEPARTMENT_ID_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@DepPerInclRegID", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = ADepPerInclID;
                    selectAllRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = null;
                    lParameter = new SqlParameter("@RegionID", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = ARegionID;
                    selectAllRegionDirectionCommand.Parameters.Add(lParameter);
                    connection.Open();
                    using (SqlDataReader RegionDirectionReader = selectAllRegionDirectionCommand.ExecuteReader())
                    {
                        if (RegionDirectionReader.Read())
                        {
                            lRegionDirectionLine =
                            new RegionDirectionInPeriodLine(int.Parse(RegionDirectionReader["ID"].ToString()),
                                                                       RegionDirectionReader["DepPerInclRegID"] == DBNull.Value ? -1 : (int)int.Parse(RegionDirectionReader["DepPerInclRegID"].ToString()),
                                                                       RegionDirectionReader["RegionID"] == DBNull.Value ? -1 : (int)int.Parse(RegionDirectionReader["RegionID"].ToString()),
                                                                       RegionDirectionReader["RegionName"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionName"],
                                                                       RegionDirectionReader["RegionSort"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionSort"],
                                                                       RegionDirectionReader["RegionCode"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionCode"],
                                                                       RegionDirectionReader["RegionDirectionName"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionDirectionName"].ToString(),
                                                                       RegionDirectionReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(RegionDirectionReader["Flag"].ToString())
                                                                                );
                        }
                        else lRegionDirectionLine = new RegionDirectionInPeriodLine();
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRegionDirectionLine;
        }
        public static List<RegionDirectionInPeriodLine> GetRegionDirectionListByDepartmentIDAndActualDate(int ADepartmentID, DateTime AActualDate)
        {
            List<RegionDirectionInPeriodLine> lRegionDirectionList = new List<RegionDirectionInPeriodLine>();
            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllRegionDirectionCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_REGION_DIRECTION_IN_PERIOD_BY_ACTUALDATE_AND_DEPARTAMENTID_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int); 
                    lParameter.IsNullable = true;
                    lParameter.Value = ADepartmentID;
                    selectAllRegionDirectionCommand.Parameters.Add(lParameter);

                    selectAllRegionDirectionCommand.Parameters.AddWithValue("@ActualDate", AActualDate.ToString(KROConstants.DateCultureSQL));

                    connection.Open();
                    using (SqlDataReader RegionDirectionReader = selectAllRegionDirectionCommand.ExecuteReader())
                    {
                        while (RegionDirectionReader.Read())
                        {
                            RegionDirectionInPeriodLine currentRole =
                            new RegionDirectionInPeriodLine(int.Parse(RegionDirectionReader["ID"].ToString()),
                                                                       RegionDirectionReader["DepPerInclRegID"] == DBNull.Value ? -1 : (int)int.Parse(RegionDirectionReader["DepPerInclRegID"].ToString()),
                                                                       RegionDirectionReader["RegionID"] == DBNull.Value ? -1 : (int)int.Parse(RegionDirectionReader["RegionID"].ToString()),
                                                                       RegionDirectionReader["RegionName"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionName"],
                                                                       RegionDirectionReader["RegionCode"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionCode"],
                                                                       RegionDirectionReader["RegionSort"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionSort"],
                                                                       RegionDirectionReader["RegionDirectionName"] == DBNull.Value ? string.Empty : (string)RegionDirectionReader["RegionDirectionName"].ToString(),
                                                                       RegionDirectionReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(RegionDirectionReader["Flag"].ToString())
                                                                                );
                            lRegionDirectionList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lRegionDirectionList;
        }
        public static bool InsertRegionDirection(RegionDirectionInPeriodLine ARegionDirectionInPeriodLine, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;

            if ((ARegionDirectionInPeriodLine.DepPerInclRegID > 0) &&
                (ARegionDirectionInPeriodLine.RegionID > 0) )
            {
                
                    SqlCommand insertRegionDirectionCommand = new SqlCommand
                    {
                        Connection = lSqlConnection,
                        CommandText = KROSQLs.INSERT_REGION_DIRECTION_IN_PERIOD_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    SqlParameter lParameter = null;

                    lParameter = new SqlParameter("@DepPerInclRegID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = ARegionDirectionInPeriodLine.DepPerInclRegID;
                    insertRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@RegionID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = ARegionDirectionInPeriodLine.RegionID;
                    insertRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@RegionDirectionName", SqlDbType.NVarChar);
                    lParameter.IsNullable = true;
                    lParameter.Value = ARegionDirectionInPeriodLine.RegionDirectionName;
                    insertRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = ARegionDirectionInPeriodLine.Flag;
                    insertRegionDirectionCommand.Parameters.Add(lParameter);

                    insertRegionDirectionCommand.Transaction = lTransaction;
                    ARegionDirectionInPeriodLine.SetID((int)insertRegionDirectionCommand.ExecuteScalar());
                    lResult = true;
            }
            return lResult;
        }
        public static bool UpdateRegionDirectionCheck(RegionDirectionInPeriodLine ARegionDirectionInPeriodLine, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;
            if ((ARegionDirectionInPeriodLine.ID > 0) &&
                (ARegionDirectionInPeriodLine.DepPerInclRegID > 0) &&
                (ARegionDirectionInPeriodLine.RegionID > 0))
            {
                 SqlCommand updateRegionDirectionCommand = new SqlCommand
                    {
                        Connection = lSqlConnection,
                        CommandText = KROSQLs.UPDATE_REGION_DIRECTION_IN_PERIOD_BY_ID_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@ID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = ARegionDirectionInPeriodLine.ID;
                    updateRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@DepPerInclRegID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = ARegionDirectionInPeriodLine.DepPerInclRegID;
                    updateRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@RegionID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = ARegionDirectionInPeriodLine.RegionID;
                    updateRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@RegionDirectionName", SqlDbType.NVarChar);
                    lParameter.IsNullable = true;
                    lParameter.Value = ARegionDirectionInPeriodLine.RegionDirectionName;
                    updateRegionDirectionCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = ARegionDirectionInPeriodLine.Flag;
                    updateRegionDirectionCommand.Parameters.Add(lParameter);

                    updateRegionDirectionCommand.Transaction = lTransaction;
                    updateRegionDirectionCommand.ExecuteNonQuery();
                    lResult = true;
            }
            return lResult;
        }
        # endregion
        #region DepartmentIDInPeriodList
        public static List<DepartmentIDInPeriodLine> GetDepartmentIDInPeriodByDepListInPeriodID(int ADepListInPeriodID)
        {
            List<DepartmentIDInPeriodLine> lDepartmentIDInPeriodList = new List<DepartmentIDInPeriodLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllDepartmentIDInPeriodCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_DEPARTMENT_ID_IN_PERIOD_BY_DEPLISTPERIODID_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@DepListInPeriodID", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = ADepListInPeriodID;
                    selectAllDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                    connection.Open();
                    using (SqlDataReader DepartmentIDInPeriodReader = selectAllDepartmentIDInPeriodCommand.ExecuteReader())
                    {
                        while (DepartmentIDInPeriodReader.Read())
                        {
                            DepartmentIDInPeriodLine currentRole =
                            new DepartmentIDInPeriodLine(int.Parse(DepartmentIDInPeriodReader["ID"].ToString()),
                                                                       DepartmentIDInPeriodReader["DepListInPeriodID"] == DBNull.Value ? -1 : (int)int.Parse(DepartmentIDInPeriodReader["DepListInPeriodID"].ToString()),
                                                                       DepartmentIDInPeriodReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(DepartmentIDInPeriodReader["DepartmentID"].ToString()),
                                                                       DepartmentIDInPeriodReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)DepartmentIDInPeriodReader["DepartmentName"],
                                                                       DepartmentIDInPeriodReader["DepartmentSort"] == DBNull.Value ? string.Empty : (string)DepartmentIDInPeriodReader["DepartmentSort"],
                                                                       DepartmentIDInPeriodReader["DepartmentCode"] == DBNull.Value ? string.Empty : (string)DepartmentIDInPeriodReader["DepartmentCode"],
                                                                       DepartmentIDInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(DepartmentIDInPeriodReader["Flag"].ToString())
                                                                                );
                            lDepartmentIDInPeriodList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lDepartmentIDInPeriodList;
        }
        public static List<DepartmentIDInPeriodLine> GetDepartmentIDInPeriodListByActualDate(DateTime AActualDate)
        {
            List<DepartmentIDInPeriodLine> lDepartmentIDInPeriodList = new List<DepartmentIDInPeriodLine>();
            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllDepartmentIDInPeriodCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_DEPARTMENT_ID_IN_PERIOD_BY_ACTUALDATE_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    selectAllDepartmentIDInPeriodCommand.Parameters.AddWithValue("@ActualDate", AActualDate.ToString(KROConstants.DateCultureSQL));

                    connection.Open();
                    using (SqlDataReader DepartmentIDInPeriodReader = selectAllDepartmentIDInPeriodCommand.ExecuteReader())
                    {
                        while (DepartmentIDInPeriodReader.Read())
                        {
                            DepartmentIDInPeriodLine currentRole =
                            new DepartmentIDInPeriodLine(int.Parse(DepartmentIDInPeriodReader["ID"].ToString()),
                                                                       DepartmentIDInPeriodReader["DepListInPeriodID"] == DBNull.Value ? -1 : (int)int.Parse(DepartmentIDInPeriodReader["DepListInPeriodID"].ToString()),
                                                                       DepartmentIDInPeriodReader["DepartmentID"] == DBNull.Value ? -1 : (int)int.Parse(DepartmentIDInPeriodReader["DepartmentID"].ToString()),
                                                                       DepartmentIDInPeriodReader["DepartmentName"] == DBNull.Value ? string.Empty : (string)DepartmentIDInPeriodReader["DepartmentName"],
                                                                       DepartmentIDInPeriodReader["DepartmentSort"] == DBNull.Value ? string.Empty : (string)DepartmentIDInPeriodReader["DepartmentSort"],
                                                                       DepartmentIDInPeriodReader["DepartmentCode"] == DBNull.Value ? string.Empty : (string)DepartmentIDInPeriodReader["DepartmentCode"],
                                                                       DepartmentIDInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(DepartmentIDInPeriodReader["Flag"].ToString())
                                                                                );
                            lDepartmentIDInPeriodList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lDepartmentIDInPeriodList;
        }
        public static bool InsertDepartmentIDInPeriod(DepartmentIDInPeriodLine ADepartmentIDInPeriodLine, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;

            if ((ADepartmentIDInPeriodLine.DepListInPeriodID > 0) &&
                (ADepartmentIDInPeriodLine.DepartmentID > 0))
            {

                SqlCommand insertDepartmentIDInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.INSERT_DEPARTMENT_ID_IN_PERIOD_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;

                lParameter = new SqlParameter("@DepListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = ADepartmentIDInPeriodLine.DepListInPeriodID;
                insertDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = ADepartmentIDInPeriodLine.DepartmentID;
                insertDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = ADepartmentIDInPeriodLine.Flag;
                insertDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                insertDepartmentIDInPeriodCommand.Transaction = lTransaction;
                ADepartmentIDInPeriodLine.SetID((int)insertDepartmentIDInPeriodCommand.ExecuteScalar());
                lResult = true;
            }
            return lResult;
        }
        public static bool UpdateDepartmentIDInPeriodCheck(DepartmentIDInPeriodLine ADepartmentIDInPeriodLine, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;
            if ((ADepartmentIDInPeriodLine.ID > 0) &&
                (ADepartmentIDInPeriodLine.DepListInPeriodID > 0) &&
                (ADepartmentIDInPeriodLine.DepartmentID > 0))
            {
                SqlCommand updateDepartmentIDInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.UPDATE_DEPARTMENT_ID_IN_PERIOD_BY_ID_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;
                lParameter = new SqlParameter("@ID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = ADepartmentIDInPeriodLine.ID;
                updateDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@DepListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = ADepartmentIDInPeriodLine.DepListInPeriodID;
                updateDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = ADepartmentIDInPeriodLine.DepartmentID;
                updateDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = ADepartmentIDInPeriodLine.Flag;
                updateDepartmentIDInPeriodCommand.Parameters.Add(lParameter);

                updateDepartmentIDInPeriodCommand.Transaction = lTransaction;
                updateDepartmentIDInPeriodCommand.ExecuteNonQuery();
                lResult = true;
            }
            return lResult;
        }
        # endregion
        # region FarmList
        public static List<FarmLine> GetFarmList(bool ADesc = true)
        {
            List<FarmLine> lFarmList = new List<FarmLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    string lSql = "";
                    if (ADesc) lSql = KROSQLs.SelectAllFarmSQLDesc; else lSql = KROSQLs.SelectAllFarmSQL;
                    SqlCommand selectAllFarmCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = lSql,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader FarmReader = selectAllFarmCommand.ExecuteReader())
                    {
                        while (FarmReader.Read())
                        {
                            FarmLine currentRole = new FarmLine(Convert.ToInt32(FarmReader.GetValue(0)),
                                                                FarmReader.GetValue(1).ToString(),
                                                                FarmReader.GetValue(2).ToString());
                            lFarmList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lFarmList;
        }
        # endregion
        # region MetricCategoryList
        public static List<MetricCategoryLine> GetMetricCategoryList(bool ADesc = true)
        {
            List<MetricCategoryLine> lMetricCategoryList = new List<MetricCategoryLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    string lSql = "";
                    if (ADesc) lSql = KROSQLs.SelectAllMetricCategorySQLDesc; else lSql = KROSQLs.SelectAllMetricCategorySQL;
                    SqlCommand selectAllMetricCategoryCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = lSql,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader MetricCategoryReader = selectAllMetricCategoryCommand.ExecuteReader())
                    {
                        while (MetricCategoryReader.Read())
                        {
                            MetricCategoryLine currentRole = new MetricCategoryLine(Convert.ToInt32(MetricCategoryReader.GetValue(0)),
                                                                                    MetricCategoryReader.GetValue(1).ToString());
                            lMetricCategoryList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lMetricCategoryList;
        }
        # endregion
        # region MetricList
        public static List<MetricLine> GetMetricList(bool AWithOutCalculatedAndText = false)
        {
            List<MetricLine> lMetricList = new List<MetricLine>();
            string lSQL;
            if (AWithOutCalculatedAndText) lSQL = KROSQLs.SELECT_ALL_METRIC_WITHOUT_CALC_AND_TEXT_SP_NAME; else lSQL = KROSQLs.SELECT_ALL_METRIC_SP_NAME;
            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllMetricCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = lSQL,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader MetricReader = selectAllMetricCommand.ExecuteReader())
                    {
                        while (MetricReader.Read())
                        {
                            MetricLine currentRole = new MetricLine(Convert.ToInt32(MetricReader.GetValue(0)),
                                                                            MetricReader["MetricName"] == DBNull.Value ? string.Empty : (string)MetricReader["MetricName"],
                                                                            MetricReader["MetricSecondName"] == DBNull.Value ? string.Empty : (string)MetricReader["MetricSecondName"],                                                                            
                                                                            MetricReader["MetricCode"] == DBNull.Value ? string.Empty : (string)MetricReader["MetricCode"],
                                                                            MetricReader["MetricSort"] == DBNull.Value ? string.Empty : (string)MetricReader["MetricSort"],
                                                                            MetricReader["MetricScreenCode"] == DBNull.Value ? string.Empty : (string)MetricReader["MetricScreenCode"],
                                                                            MetricReader["FarmID"] == DBNull.Value ? 0 : (Int32)Int32.Parse(MetricReader["FarmID"].ToString()),
                                                                            MetricReader["FarmName"] == DBNull.Value ? string.Empty : (string)MetricReader["FarmName"],
                                                                            MetricReader["FarmSort"] == DBNull.Value ? string.Empty : (string)MetricReader["FarmSort"],
                                                                            MetricReader["MetricCategoryID"] == DBNull.Value ? 0 : (Int32)Int32.Parse(MetricReader["MetricCategoryID"].ToString()),
                                                                            MetricReader["MetricCategoryName"] == DBNull.Value ? string.Empty : (string)MetricReader["MetricCategoryName"],
                                                                            MetricReader["Flag"] == DBNull.Value ? 0 : (long)long.Parse(MetricReader["Flag"].ToString())
                                                                            );
                            lMetricList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lMetricList;
        }
        public static bool InsertMetric(MetricLine AMetricLine)
        {
            bool lResult = false;


            if ((AMetricLine.MetricName.Length > 0) &&
                (AMetricLine.MetricSort.Length > 0))
            {
                //подготовка данных подрежем первые 10 символов у AMetricCode и AMetricSort и подправим AMetricCategory
                if (AMetricLine.MetricCode.Length > KROConstants.MaxLegthCodeSort) AMetricLine.MetricCode = AMetricLine.MetricCode.Substring(0, KROConstants.MaxLegthCodeSort);
                if (AMetricLine.MetricSort.Length > KROConstants.MaxLegthCodeSort) AMetricLine.MetricSort = AMetricLine.MetricSort.Substring(0, KROConstants.MaxLegthCodeSort);
                if (AMetricLine.MetricScreenCode.Length > KROConstants.MaxLegthCodeSort) AMetricLine.MetricSort = AMetricLine.MetricSort.Substring(0, KROConstants.MaxLegthCodeSort);
                if (AMetricLine.Flag < 0) AMetricLine.SetFlag(0);
                //конец подготовки данных
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertMetricCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_METRIC_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@MetricName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricLine.MetricName;
                        insertMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricSecondName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        if (AMetricLine.MetricSecondName.Length > 0)
                        lParameter.Value = AMetricLine.MetricSecondName; else lParameter.Value = AMetricLine.MetricName;
                        insertMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricLine.MetricCode;
                        insertMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricSort", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricLine.MetricSort;
                        insertMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricScreenCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        lParameter.Value = AMetricLine.MetricScreenCode;
                        insertMetricCommand.Parameters.Add(lParameter);


                        lParameter = new SqlParameter("@FarmID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        if (AMetricLine.MetricFarmID > 0) lParameter.Value = AMetricLine.MetricFarmID; else lParameter.Value = DBNull.Value;
                        insertMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricCategoryID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        if (AMetricLine.MetricFarmID > 0) lParameter.Value = AMetricLine.MetricCategoryID; else lParameter.Value = 1;
                        insertMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.BigInt);
                        lParameter.IsNullable = true;
                        lParameter.Value = AMetricLine.Flag;
                        insertMetricCommand.Parameters.Add(lParameter);

                        connection.Open();
                        AMetricLine.SetMetricID((int)insertMetricCommand.ExecuteScalar());
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool UpdateMetricCheck(MetricLine AMetricLine)
        {
            bool lResult = false;
            if ((AMetricLine.MetricID > 0) &&
                (AMetricLine.MetricName.Length > 0) &&
                (AMetricLine.MetricSort.Length > 0))
            {
                //подготовка данных подрежем первые 10 символов у AMetricCode и AMetricSort и подправим AMetricCategory
                if (AMetricLine.MetricCode.Length > KROConstants.MaxLegthCodeSort) AMetricLine.MetricCode = AMetricLine.MetricCode.Substring(0, KROConstants.MaxLegthCodeSort);
                if (AMetricLine.MetricSort.Length > KROConstants.MaxLegthCodeSort) AMetricLine.MetricSort = AMetricLine.MetricSort.Substring(0, KROConstants.MaxLegthCodeSort);
                if (AMetricLine.MetricScreenCode.Length > KROConstants.MaxLegthCodeSort) AMetricLine.MetricSort = AMetricLine.MetricSort.Substring(0, KROConstants.MaxLegthCodeSort);
                if (AMetricLine.Flag < 0) AMetricLine.SetFlag(0);
                //конец подготовки данных
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateMetricCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_METRIC_BY_ID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricLine.MetricID;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricLine.MetricName;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricSecondName", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        if (AMetricLine.MetricSecondName.Length > 0)
                            lParameter.Value = AMetricLine.MetricSecondName;
                        else lParameter.Value = AMetricLine.MetricName;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricLine.MetricCode;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricSort", SqlDbType.NVarChar);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricLine.MetricSort;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricScreenCode", SqlDbType.NVarChar);
                        lParameter.IsNullable = true;
                        lParameter.Value = AMetricLine.MetricScreenCode;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@FarmID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        if (AMetricLine.MetricFarmID > 0) lParameter.Value = AMetricLine.MetricFarmID; else lParameter.Value = DBNull.Value;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MetricCategoryID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        if (AMetricLine.MetricFarmID > 0) lParameter.Value = AMetricLine.MetricCategoryID; else lParameter.Value = 1;
                        updateMetricCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.BigInt);
                        lParameter.IsNullable = true;
                        lParameter.Value = AMetricLine.Flag;
                        updateMetricCommand.Parameters.Add(lParameter);

                        connection.Open();
                        updateMetricCommand.ExecuteNonQuery();
                        lResult = true;
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        # endregion
        # region MetricListInPeriodList
        public static List<MetricListInPeriodLine> GetMetricListInPeriodList()
        {
            List<MetricListInPeriodLine> lMetricListInPeriodList = new List<MetricListInPeriodLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllMetricListInPeriodCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_METRICLISTINPERIOD_SP_NAME,
                        CommandType = CommandType.Text
                    };
                    connection.Open();
                    using (SqlDataReader MetricListInPeriodReader = selectAllMetricListInPeriodCommand.ExecuteReader())
                    {
                        while (MetricListInPeriodReader.Read())
                        {
                            MetricListInPeriodLine currentRole = new MetricListInPeriodLine(int.Parse(MetricListInPeriodReader["ID"].ToString()),
                                                                          MetricListInPeriodReader["MounthID"] == DBNull.Value ? -1 : (int)int.Parse(MetricListInPeriodReader["MounthID"].ToString()),
                                                                          MetricListInPeriodReader["MonthName"] == DBNull.Value ? string.Empty : (string)MetricListInPeriodReader["MonthName"],
                                                                          MetricListInPeriodReader["YearID"] == DBNull.Value ? -1 : (int)int.Parse(MetricListInPeriodReader["YearID"].ToString()),
                                                                          MetricListInPeriodReader["YearName"] == DBNull.Value ? string.Empty : (string)MetricListInPeriodReader["YearName"].ToString(),
                                                                          MetricListInPeriodReader["DateIn"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(MetricListInPeriodReader["DateIn"].ToString()),
                                                                          MetricListInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(MetricListInPeriodReader["Flag"].ToString())
                                                                                  );
                            lMetricListInPeriodList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lMetricListInPeriodList;
        }
        public static List<MetricListInPeriodLine> GetMetricListInPeriodListByActualDate(DateTime AActualDate)
        {
            List<MetricListInPeriodLine> lMetricListInPeriodList = new List<MetricListInPeriodLine>();
            if (AActualDate != null)
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllMetricListInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_METRICLISTINPERIOD_BY_ACTUALDATE_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        selectAllMetricListInPeriodCommand.Parameters.AddWithValue("@ActualDate", AActualDate.ToString(KROConstants.DateCultureSQL));

                        connection.Open();
                        using (SqlDataReader MetricListInPeriodReader = selectAllMetricListInPeriodCommand.ExecuteReader())
                        {
                            while (MetricListInPeriodReader.Read())
                            {
                                MetricListInPeriodLine currentRole = new MetricListInPeriodLine(int.Parse(MetricListInPeriodReader["ID"].ToString()),
                                                                              MetricListInPeriodReader["MounthID"] == DBNull.Value ? -1 : (int)int.Parse(MetricListInPeriodReader["MounthID"].ToString()),
                                                                              MetricListInPeriodReader["MonthName"] == DBNull.Value ? string.Empty : (string)MetricListInPeriodReader["MonthName"],
                                                                              MetricListInPeriodReader["YearID"] == DBNull.Value ? -1 : (int)int.Parse(MetricListInPeriodReader["YearID"].ToString()),
                                                                              MetricListInPeriodReader["YearName"] == DBNull.Value ? string.Empty : (string)MetricListInPeriodReader["YearName"].ToString(),
                                                                              MetricListInPeriodReader["DateIn"] == DBNull.Value ? (DateTime)DateTime.ParseExact(KROConstants.DatePasswordDefault, KROConstants.DateCultureDefault, CultureInfo.InvariantCulture) : (DateTime)DateTime.Parse(MetricListInPeriodReader["DateIn"].ToString()),
                                                                              MetricListInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(MetricListInPeriodReader["Flag"].ToString())
                                                                                      );
                                lMetricListInPeriodList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lMetricListInPeriodList;
        }
        public static bool InsertMetricListInPeriod(MetricListInPeriodLine AMetricListInPeriodLine, List<MetricIDInPeriodLine> AMetricIDInPeriodList, List<MetricFunctionLine> AFullListOldMetricFunction)
        {
            bool lResult = false;

            if ((AMetricListInPeriodLine.MounthID > 0) &&
                (AMetricListInPeriodLine.YearID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand insertMetricListInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.INSERT_METRICLISTINPERIOD_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@MonthIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricListInPeriodLine.MounthID;
                        insertMetricListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricListInPeriodLine.YearID;
                        insertMetricListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = AMetricListInPeriodLine.Flag;
                        insertMetricListInPeriodCommand.Parameters.Add(lParameter);

                        connection.Open();
                        SqlTransaction lTransaction = connection.BeginTransaction();
                        try
                        {
                            insertMetricListInPeriodCommand.Transaction = lTransaction;
                            AMetricListInPeriodLine.SetID((int)insertMetricListInPeriodCommand.ExecuteScalar());
                            // успешно сохранили новую запись 
                            // пошли сохранять остальные параметры
                            foreach (MetricIDInPeriodLine lMetricIDInPeriodLine in AMetricIDInPeriodList)
                            {
                                lMetricIDInPeriodLine.MetricListInPeriodID = AMetricListInPeriodLine.ID;
                                if (lMetricIDInPeriodLine.ID > 0) UpdateMetricIDInPeriodCheck(lMetricIDInPeriodLine, connection, lTransaction);
                                else
                                {
                                    try {
                                       InsertMetricIDInPeriod(lMetricIDInPeriodLine, connection, lTransaction);
                                    }
                                    catch (Exception expinsert)
                                    {
                                        lMetricIDInPeriodLine.SetID(-1);
                                        throw expinsert;
                                    }
                                }
                            }
                            InsertAllMetricFunctionOnlyFirstSaveMetricListInPeriod(AMetricListInPeriodLine, AMetricIDInPeriodList, AFullListOldMetricFunction, connection, lTransaction);
                            lTransaction.Commit();
                            connection.Close();
                            lResult = true;
                        }
                        catch
                        {
                            lTransaction.Rollback();
                            throw;
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        } 
        // Будем вызывать один раз чтобы протащить все функции пакетом  (съэкономим кучу циклов)
        public static bool InsertAllMetricFunctionOnlyFirstSaveMetricListInPeriod(MetricListInPeriodLine AMetricListInPeriodLine, //NewMetricListID
                                                                                  List<MetricIDInPeriodLine> AMetricIDInPeriodList, // all new metricID wish parameter
                                                                                  List<MetricFunctionLine> AFullListOldMetricFunction, // all old function list
                                                                                  SqlConnection ASqlConnection, SqlTransaction ATransaction)
        {  
            bool lResult = false;
            // проверим а есть ли с чем вообще работать  
            if ((AFullListOldMetricFunction.Count > 0) && // есть ли вообще старые функции
                (AMetricListInPeriodLine.ID > 0) && // есть ли вообще старые функции
                (AMetricIDInPeriodList.Count > 0)) // есть ли список новых показателей
            {   //начинаем подготовку вставки функций
                List<MetricIDInPeriodLine> lAllCheckedMetricIDInPeriodLine = AMetricIDInPeriodList.FindAll(lList => lList.MetricIDCheckedInPeriod);
                List<MetricIDInPeriodLine> lAllCheckedCalculationMetricIDInPeriodLine = AMetricIDInPeriodList.FindAll(lList => (lList.MetricIDCheckedInPeriod && ((lList.MetricFlag & KROFlag.flMetricCalculations) == KROFlag.flMetricCalculations)));
                if (lAllCheckedCalculationMetricIDInPeriodLine.Count > 0) //погнали что-то делать если есть выбранные вычисляемые поля 
                {
                    foreach (MetricIDInPeriodLine lNewClalcCheckedMetric in lAllCheckedCalculationMetricIDInPeriodLine)
                    {
                        List<MetricFunctionLine> lFindOldFunction = AFullListOldMetricFunction.FindAll(lFunctionList => lFunctionList.CalcMetricID == lNewClalcCheckedMetric.MetricID);
                        // теперь проверим а нашли-ли чего и вставлять будем только выбранные элементы
                        if (lFindOldFunction.Count > 0)
                        {
                            foreach (MetricFunctionLine lOldMetricFunction in lFindOldFunction)
                            {
                                if (lAllCheckedMetricIDInPeriodLine.FindIndex(lCheckedMetricIDLine => lCheckedMetricIDLine.MetricID == lOldMetricFunction.SumMetricID) >= 0)
                                { //можно добавлять
                                    MetricFunctionInPeriodLine lFunction = new MetricFunctionInPeriodLine();
                                    lFunction.MetricListInPeriodID = AMetricListInPeriodLine.ID;
                                    lFunction.CalcMetricID = lNewClalcCheckedMetric.MetricID;
                                    lFunction.SumMetricID = lOldMetricFunction.SumMetricID;
                                    lFunction.SumMetricDK = lOldMetricFunction.DK;
                                    InsertMetricFunctionInPeriod(lFunction, ASqlConnection, ATransaction);
                                }
                            }
                        }
                    }
                    lResult = true;
                }
            }
            return lResult;
        }
        public static bool UpdateMetricListInPeriodCheck(MetricListInPeriodLine AMetricListInPeriodLine, List<MetricIDInPeriodLine> AMetricIDInPeriodList)
        {
            bool lResult = false;
            if ((AMetricListInPeriodLine.ID > 0) &&
                (AMetricListInPeriodLine.MounthID > 0) &&
                (AMetricListInPeriodLine.YearID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand updateMetricListInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.UPDATE_METRICLISTINPERIOD_BY_ID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricListInPeriodLine.ID;
                        updateMetricListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@MonthIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricListInPeriodLine.MounthID;
                        updateMetricListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearIN", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricListInPeriodLine.YearID;
                        updateMetricListInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = AMetricListInPeriodLine.Flag;
                        updateMetricListInPeriodCommand.Parameters.Add(lParameter);

                        connection.Open();
                        SqlTransaction lTransaction = connection.BeginTransaction();
                        try
                        {
                            updateMetricListInPeriodCommand.Transaction = lTransaction;
                            updateMetricListInPeriodCommand.ExecuteNonQuery();
                            // пошли сохранять остальные параметры
                            foreach (MetricIDInPeriodLine lMetricIDInPeriodLine in AMetricIDInPeriodList)
                            {
                                lMetricIDInPeriodLine.MetricListInPeriodID = AMetricListInPeriodLine.ID;
                                if (lMetricIDInPeriodLine.ID > 0) UpdateMetricIDInPeriodCheck(lMetricIDInPeriodLine, connection, lTransaction);
                                else
                                {
                                    try
                                    {
                                        InsertMetricIDInPeriod(lMetricIDInPeriodLine, connection, lTransaction);
                                    }
                                    catch (Exception expinsert)
                                    {
                                        lMetricIDInPeriodLine.SetID(-1);
                                        throw expinsert;
                                    }
                                }
                            }
                            lTransaction.Commit();
                            connection.Close();
                            lResult = true;
                        }
                        catch
                        {
                            lTransaction.Rollback();
                            throw;
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static void AddChangeMetricListInPeriodLineInSession(HttpSessionState ASession, MetricListInPeriodLine AMetricListInPeriodLine)
        {
            ASession[KROConstants.SESSION_CHANGE_METRICLISTINPERIOD_LINE] = AMetricListInPeriodLine;
        }
        public static MetricListInPeriodLine GetChangeMetricListInPeriodLineFromSession(HttpSessionState ASession)
        {
            MetricListInPeriodLine lMetricListInPeriodLine = new MetricListInPeriodLine();
            if ((ASession != null) && (ASession.Count > 0))
            {
                try
                {   //попробуем получить изменяемый список покказателей в периоде
                    lMetricListInPeriodLine = (MetricListInPeriodLine)ASession[KROConstants.SESSION_CHANGE_METRICLISTINPERIOD_LINE];
                }
                catch { }
            }
            return lMetricListInPeriodLine;
        }
        # endregion
        #region MetricIDInPeriodList 
        public static bool GetAlreadyExistFactDataLastPeriod(int AMounthID, int AYearID)
        {
            bool lResult = true;

            if ((AMounthID > 0) &&
                (AYearID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand GetAlreadyExistFactDatacommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.GET_ALREADY_EXIST_FACT_DATA_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@MounthID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMounthID;
                        GetAlreadyExistFactDatacommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@YearID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AYearID;
                        GetAlreadyExistFactDatacommand.Parameters.Add(lParameter);

                        connection.Open();
                        lResult = ((int) GetAlreadyExistFactDatacommand.ExecuteScalar() == 1 ? true : false);
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static List<MetricIDInPeriodLine> GetMetricIDInPeriodByMetricListInPeriodID(int AMetricListInPeriodID)
        {
            List<MetricIDInPeriodLine> lMetricIDInPeriodList = new List<MetricIDInPeriodLine>();

            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllMetricIDInPeriodCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_METRIC_ID_IN_PERIOD_BY_METRICLISTPERIODID_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };
                    SqlParameter lParameter = null;
                    lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                    lParameter.IsNullable = true;
                    lParameter.Value = AMetricListInPeriodID;
                    selectAllMetricIDInPeriodCommand.Parameters.Add(lParameter);

                    connection.Open();
                    using (SqlDataReader MetricIDInPeriodReader = selectAllMetricIDInPeriodCommand.ExecuteReader())
                    {
                        while (MetricIDInPeriodReader.Read())
                        {
                            MetricIDInPeriodLine currentRole =
                            new MetricIDInPeriodLine(int.Parse(MetricIDInPeriodReader["ID"].ToString()),
                                                                       MetricIDInPeriodReader["MetricListInPeriodID"] == DBNull.Value ? -1 : (int)int.Parse(MetricIDInPeriodReader["MetricListInPeriodID"].ToString()),
                                                                       MetricIDInPeriodReader["MetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricIDInPeriodReader["MetricID"].ToString()),
                                                                       MetricIDInPeriodReader["MetricName"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricName"],
                                                                       MetricIDInPeriodReader["MetricSort"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricSort"],
                                                                       MetricIDInPeriodReader["MetricCode"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricCode"],
                                                                       MetricIDInPeriodReader["MetricScreenCode"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricScreenCode"],
                                                                       MetricIDInPeriodReader["MetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricIDInPeriodReader["MetricFlag"].ToString()),
                                                                       MetricIDInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(MetricIDInPeriodReader["Flag"].ToString())
                                                                                );
                            lMetricIDInPeriodList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lMetricIDInPeriodList;
        }
        public static List<MetricIDInPeriodLine> GetMetricIDInPeriodListByActualDate(DateTime AActualDate)
        {
            List<MetricIDInPeriodLine> lMetricIDInPeriodList = new List<MetricIDInPeriodLine>();
            using (
                SqlConnection connection =
                    new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
            {
                try
                {
                    SqlCommand selectAllMetricIDInPeriodCommand = new SqlCommand
                    {
                        Connection = connection,
                        CommandText = KROSQLs.SELECT_ALL_METRIC_ID_IN_PERIOD_BY_ACTUALDATE_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    selectAllMetricIDInPeriodCommand.Parameters.AddWithValue("@ActualDate", AActualDate.ToString(KROConstants.DateCultureSQL));

                    connection.Open();
                    using (SqlDataReader MetricIDInPeriodReader = selectAllMetricIDInPeriodCommand.ExecuteReader())
                    {
                        while (MetricIDInPeriodReader.Read())
                        {
                            MetricIDInPeriodLine currentRole =
                            new MetricIDInPeriodLine(int.Parse(MetricIDInPeriodReader["ID"].ToString()),
                                                                       MetricIDInPeriodReader["MetricListInPeriodID"] == DBNull.Value ? -1 : (int)int.Parse(MetricIDInPeriodReader["MetricListInPeriodID"].ToString()),
                                                                       MetricIDInPeriodReader["MetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricIDInPeriodReader["MetricID"].ToString()),
                                                                       MetricIDInPeriodReader["MetricName"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricName"],
                                                                       MetricIDInPeriodReader["MetricSort"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricSort"],
                                                                       MetricIDInPeriodReader["MetricCode"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricCode"],
                                                                       MetricIDInPeriodReader["MetricScreenCode"] == DBNull.Value ? string.Empty : (string)MetricIDInPeriodReader["MetricScreenCode"],
                                                                       MetricIDInPeriodReader["MetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricIDInPeriodReader["MetricFlag"].ToString()),
                                                                       MetricIDInPeriodReader["Flag"] == DBNull.Value ? 0 : (int)int.Parse(MetricIDInPeriodReader["Flag"].ToString())
                                                                                );
                            lMetricIDInPeriodList.Add(currentRole);
                        }
                    }
                }
                finally { if (connection != null) connection.Close(); }
            }
            return lMetricIDInPeriodList;
        }
        public static bool InsertMetricIDInPeriod(MetricIDInPeriodLine AMetricIDInPeriodLine, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;

            if ((AMetricIDInPeriodLine.MetricListInPeriodID > 0) &&
                (AMetricIDInPeriodLine.MetricID > 0))
            {

                SqlCommand insertMetricIDInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.INSERT_METRIC_ID_IN_PERIOD_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;

                lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricIDInPeriodLine.MetricListInPeriodID;
                insertMetricIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@MetricID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricIDInPeriodLine.MetricID;
                insertMetricIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = AMetricIDInPeriodLine.Flag;
                insertMetricIDInPeriodCommand.Parameters.Add(lParameter);

                insertMetricIDInPeriodCommand.Transaction = lTransaction;
                AMetricIDInPeriodLine.SetID((int)insertMetricIDInPeriodCommand.ExecuteScalar());
                lResult = true;
            }
            return lResult;
        }
        public static bool UpdateMetricIDInPeriodCheck(MetricIDInPeriodLine AMetricIDInPeriodLine, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;
            if ((AMetricIDInPeriodLine.ID > 0) &&
                (AMetricIDInPeriodLine.MetricListInPeriodID > 0) &&
                (AMetricIDInPeriodLine.MetricID > 0))
            {
                SqlCommand updateMetricIDInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.UPDATE_METRIC_ID_IN_PERIOD_BY_ID_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;
                lParameter = new SqlParameter("@ID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricIDInPeriodLine.ID;
                updateMetricIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricIDInPeriodLine.MetricListInPeriodID;
                updateMetricIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@MetricID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricIDInPeriodLine.MetricID;
                updateMetricIDInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@Flag", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = AMetricIDInPeriodLine.Flag;
                updateMetricIDInPeriodCommand.Parameters.Add(lParameter);

                updateMetricIDInPeriodCommand.Transaction = lTransaction;
                updateMetricIDInPeriodCommand.ExecuteNonQuery();
                lResult = true;
            }
            return lResult;
        }
        # endregion
        #region MetricFunctionInPeriodList
        public static List<MetricFunctionInPeriodLine> GetMetricFunctionInPeriodByMetricListInPeriodIDAndCalcMetricID(int AMetricListInPeriodID, int ACalcMetricID)
        {   // получается что данный запрос возвращает только выбранные показатели в формуле поэтому при формировании List checked выставляем в true
            List<MetricFunctionInPeriodLine> lMetricFunctionInPeriodList = new List<MetricFunctionInPeriodLine>();
            if ((AMetricListInPeriodID > 0) && (ACalcMetricID>0)) // только при наличии входных параметров
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllMetricFunctionInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_METRICFUNCTIONINPERIOD_BY_METRICLISTPERIODID_AND_CALCMETRICID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = AMetricListInPeriodID;
                        selectAllMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                        lParameter = new SqlParameter("@CalculatedMetricID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ACalcMetricID;
                        selectAllMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                        connection.Open();
                        using (SqlDataReader MetricFunctionInPeriodReader = selectAllMetricFunctionInPeriodCommand.ExecuteReader())
                        {
                            while (MetricFunctionInPeriodReader.Read())
                            {
                                MetricFunctionInPeriodLine currentRole =
                                new MetricFunctionInPeriodLine(int.Parse(MetricFunctionInPeriodReader["ID"].ToString()),
                                                                           MetricFunctionInPeriodReader["MetricListInPeriodID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionInPeriodReader["MetricListInPeriodID"].ToString()),
                                                                           MetricFunctionInPeriodReader["CalculatedMetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionInPeriodReader["CalculatedMetricID"].ToString()),
                                                                           MetricFunctionInPeriodReader["CalculatedMetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricFunctionInPeriodReader["CalculatedMetricFlag"].ToString()),
                                                                           MetricFunctionInPeriodReader["SumMetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionInPeriodReader["SumMetricID"].ToString()),
                                                                           MetricFunctionInPeriodReader["SumMetricName"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricName"],
                                                                           MetricFunctionInPeriodReader["SumMetricSort"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricSort"],
                                                                           MetricFunctionInPeriodReader["SumMetricCode"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricCode"],
                                                                           MetricFunctionInPeriodReader["SumMetricScreenCode"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricScreenCode"],
                                                                           MetricFunctionInPeriodReader["SumMetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricFunctionInPeriodReader["SumMetricFlag"].ToString()),
                                                                           MetricFunctionInPeriodReader["DK"] == DBNull.Value ? 0 : (int)int.Parse(MetricFunctionInPeriodReader["DK"].ToString()),
                                                                           true);
                                lMetricFunctionInPeriodList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lMetricFunctionInPeriodList;
        }
        public static List<MetricFunctionInPeriodLine> GetMetricFunctionInPeriodListByActualDateAndCalcMetricID(DateTime AActualDate, int ACalcMetricID)
        {   // получается что данный запрос возвращает только выбранные показатели в формуле поэтому при формировании List checked выставляем в true
            List<MetricFunctionInPeriodLine> lMetricFunctionInPeriodList = new List<MetricFunctionInPeriodLine>();
            if (ACalcMetricID > 0) // только при наличии расчетного показателя
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllMetricFunctionInPeriodCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_METRICFUNCTIONINPERIOD_BY_ACTUALDATE_AND_CALCMETRICID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@CalculatedMetricID", SqlDbType.Int);
                        lParameter.IsNullable = true;
                        lParameter.Value = ACalcMetricID;
                        selectAllMetricFunctionInPeriodCommand.Parameters.Add(lParameter);
                        selectAllMetricFunctionInPeriodCommand.Parameters.AddWithValue("@ActualDate", AActualDate.ToString(KROConstants.DateCultureSQL));

                        connection.Open();
                        using (SqlDataReader MetricFunctionInPeriodReader = selectAllMetricFunctionInPeriodCommand.ExecuteReader())
                        {
                            while (MetricFunctionInPeriodReader.Read())
                            {
                                MetricFunctionInPeriodLine currentRole =
                                new MetricFunctionInPeriodLine(int.Parse(MetricFunctionInPeriodReader["ID"].ToString()),
                                                                           MetricFunctionInPeriodReader["MetricListInPeriodID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionInPeriodReader["MetricListInPeriodID"].ToString()),
                                                                           MetricFunctionInPeriodReader["CalculatedMetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionInPeriodReader["CalculatedMetricID"].ToString()),
                                                                           MetricFunctionInPeriodReader["CalculatedMetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricFunctionInPeriodReader["CalculatedMetricFlag"].ToString()),
                                                                           MetricFunctionInPeriodReader["SumMetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionInPeriodReader["SumMetricID"].ToString()),
                                                                           MetricFunctionInPeriodReader["SumMetricName"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricName"],
                                                                           MetricFunctionInPeriodReader["SumMetricSort"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricSort"],
                                                                           MetricFunctionInPeriodReader["SumMetricCode"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricCode"],
                                                                           MetricFunctionInPeriodReader["SumMetricScreenCode"] == DBNull.Value ? string.Empty : (string)MetricFunctionInPeriodReader["SumMetricScreenCode"],
                                                                           MetricFunctionInPeriodReader["SumMetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricFunctionInPeriodReader["SumMetricFlag"].ToString()),
                                                                           MetricFunctionInPeriodReader["DK"] == DBNull.Value ? 0 : (int)int.Parse(MetricFunctionInPeriodReader["DK"].ToString()),
                                                                           true);
                                lMetricFunctionInPeriodList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lMetricFunctionInPeriodList;
        }
        public static bool InsertMetricFunctionInPeriod(MetricFunctionInPeriodLine AMetricFunctionInPeriodLine,  SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;

            if ((AMetricFunctionInPeriodLine.MetricListInPeriodID > 0) &&
                (AMetricFunctionInPeriodLine.CalcMetricID > 0) &&
                (AMetricFunctionInPeriodLine.SumMetricID > 0))
            {

                SqlCommand insertMetricFunctionInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.INSERT_METRICFUNCTIONINPERIOD_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;

                lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricFunctionInPeriodLine.MetricListInPeriodID;
                insertMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@CalculatedMetricID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricFunctionInPeriodLine.CalcMetricID;
                insertMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@SumMetricID", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = AMetricFunctionInPeriodLine.SumMetricID;
                insertMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@DK", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = AMetricFunctionInPeriodLine.SumMetricDK;
                insertMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                insertMetricFunctionInPeriodCommand.Transaction = lTransaction;
                AMetricFunctionInPeriodLine.SetID((int)insertMetricFunctionInPeriodCommand.ExecuteScalar());
                lResult = true;
            }
            return lResult;
        }
        public static bool UpdateMetricFunctionInPeriodCheck(MetricFunctionInPeriodLine AMetricFunctionInPeriodLine, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;
            if ((AMetricFunctionInPeriodLine.ID > 0) &&
                (AMetricFunctionInPeriodLine.MetricListInPeriodID > 0) &&
                (AMetricFunctionInPeriodLine.CalcMetricID > 0) &&
                (AMetricFunctionInPeriodLine.SumMetricID > 0))
            {
                SqlCommand updateMetricFunctionInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.UPDATE_METRICFUNCTIONINPERIOD_BY_ID_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;
                lParameter = new SqlParameter("@ID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricFunctionInPeriodLine.ID;
                updateMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricFunctionInPeriodLine.MetricListInPeriodID;
                updateMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@CalculatedMetricID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricFunctionInPeriodLine.CalcMetricID;
                updateMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@SumMetricID", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = AMetricFunctionInPeriodLine.SumMetricID;
                updateMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@DK", SqlDbType.Int);
                lParameter.IsNullable = true;
                lParameter.Value = AMetricFunctionInPeriodLine.SumMetricDK;
                updateMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                updateMetricFunctionInPeriodCommand.Transaction = lTransaction;
                updateMetricFunctionInPeriodCommand.ExecuteNonQuery();
                lResult = true;
            }
            return lResult;
        }
        public static bool UpdateMetricFunctionListInPeriodCheck(MetricFunctionInPeriodBaseLine AMetricFunctionInPeriodBaseLine, List<MetricFunctionInPeriodLine> AMetricFunctionInPeriodList)
        {
            bool lResult = false;
            if ((AMetricFunctionInPeriodBaseLine.ID > 0) &&
                (AMetricFunctionInPeriodBaseLine.CalcMetricID > 0))
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    { 
                        connection.Open();
                        SqlTransaction lTransaction = connection.BeginTransaction();
                        try
                        {   //подчистим старую функцию
                            ClearMetricFunctionInPeriodByMetricListInPeroidIDAndCalcMetricID(AMetricFunctionInPeriodBaseLine.ID, AMetricFunctionInPeriodBaseLine.CalcMetricID, connection, lTransaction);
                            // пошли сохранять новое выражение
                            foreach (MetricFunctionInPeriodLine lMetricFunctionInPeriodLine in AMetricFunctionInPeriodList)
                            {   //сохраним только выбранные показатели
                                if (lMetricFunctionInPeriodLine.SumMetricIDCheckedInFunction)
                                {
                                    lMetricFunctionInPeriodLine.MetricListInPeriodID = AMetricFunctionInPeriodBaseLine.ID;
                                    lMetricFunctionInPeriodLine.CalcMetricID = AMetricFunctionInPeriodBaseLine.CalcMetricID;
                                    InsertMetricFunctionInPeriod(lMetricFunctionInPeriodLine, connection, lTransaction);
                                }
                                
                            }
                            lTransaction.Commit();
                            connection.Close();
                            lResult = true;
                        }
                        catch
                        {
                            lTransaction.Rollback();
                            throw;
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lResult;
        }
        public static bool DeleteMetricFunctionInPeriodByID(int AMetricFunctionInPeriodID, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;

            if (AMetricFunctionInPeriodID > 0)
            {

                SqlCommand deleteMetricFunctionInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.DELETE_METRICFUNCTIONINPERIOD_BY_ID_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;

                lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricFunctionInPeriodID;
                deleteMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                deleteMetricFunctionInPeriodCommand.Transaction = lTransaction;
                deleteMetricFunctionInPeriodCommand.ExecuteNonQuery();
                lResult = true;
            }
            return lResult;
        }
        public static bool ClearMetricFunctionInPeriodByMetricListInPeroidID(int AMetricListInPeroidID, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;

            if (AMetricListInPeroidID > 0)
            {

                SqlCommand ClearMetricFunctionInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.CLEAR_METRICFUNCTIONINPERIOD_BY_METRICLISTPERIODID_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };
                SqlParameter lParameter = null;

                lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricListInPeroidID;
                ClearMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                ClearMetricFunctionInPeriodCommand.Transaction = lTransaction;
                ClearMetricFunctionInPeriodCommand.ExecuteNonQuery();
                lResult = true;
            }
            return lResult;
        }
        public static bool ClearMetricFunctionInPeriodByMetricListInPeroidIDAndCalcMetricID(int AMetricFunctionInPeriodID, int ACalcMetricID, SqlConnection lSqlConnection, SqlTransaction lTransaction)
        {
            bool lResult = false;

            if ((AMetricFunctionInPeriodID > 0) &&
               (ACalcMetricID>0))
            {

                SqlCommand ClearMetricFunctionInPeriodCommand = new SqlCommand
                {
                    Connection = lSqlConnection,
                    CommandText = KROSQLs.CLEAR_METRICFUNCTIONINPERIOD_BY_METRICLISTPERIODID_AND_CALCMETRICID_SP_NAME,
                    CommandType = CommandType.StoredProcedure
                };

                SqlParameter lParameter = null;

                lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = AMetricFunctionInPeriodID;
                ClearMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                lParameter = new SqlParameter("@CalculatedMetricID", SqlDbType.Int);
                lParameter.IsNullable = false;
                lParameter.Value = ACalcMetricID;
                ClearMetricFunctionInPeriodCommand.Parameters.Add(lParameter);

                ClearMetricFunctionInPeriodCommand.Transaction = lTransaction;
                ClearMetricFunctionInPeriodCommand.ExecuteNonQuery();
                lResult = true;
            }
            return lResult;
        }
        # endregion        
        #region MetricsReportList
        public static List<DinamicMetricReportLine> GetMetricReportList(ReportsParameterLine AReportsParameterLine, List<RegionLine> ADinamicRegionReportList)
        {
            List<DinamicMetricReportLine> lMetricReportList = new List<DinamicMetricReportLine>();
            if (AReportsParameterLine.ReportParameterID > 0)
            {
                string lSQL;
                lSQL = KROSQLs.SELECT_ALL_ONLYCHECKEDMETRICIDINPERIOD_BY_REPORTPARAMETERID_SP_NAME;
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllMetricReportListCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = lSQL,
                            CommandType = CommandType.StoredProcedure
                        };

                        SqlParameter lParameter = null;

                        lParameter = new SqlParameter("@ReportParameterID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportsParameterLine.ReportParameterID;
                        selectAllMetricReportListCommand.Parameters.Add(lParameter);

                        connection.Open();
                        using (SqlDataReader MetricReportReader = selectAllMetricReportListCommand.ExecuteReader())
                        {
                            while (MetricReportReader.Read())
                            {   //получили объект строки показателя отчета
                                DinamicMetricReportLine currentMetricReport =
                                    new DinamicMetricReportLine(long.Parse(MetricReportReader["MetricID"].ToString()),
                                                                MetricReportReader["MetricName"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricName"],
                                                                MetricReportReader["MetricCode"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricCode"],
                                                                MetricReportReader["MetricSort"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricSort"],
                                                                MetricReportReader["MetricScreenCode"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricScreenCode"],
                                                                MetricReportReader["MetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricReportReader["MetricFlag"].ToString())
                                                                );
                                // теперь необходимо получить объекты значений региональных дирекций и пришить к показателю
                                foreach (RegionLine lRegionLine in ADinamicRegionReportList)
                                {
                                   currentMetricReport.MetricRegionFactList.Add(SelectOnlyOneFactRow(AReportsParameterLine, currentMetricReport, lRegionLine));
                                }
                                lMetricReportList.Add(currentMetricReport); // добавили его в лист показателей уже польностью готовый
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lMetricReportList;
        }
        public static List<DinamicMetricReportLine> GetMetricReportListFull(ReportsParameterLine AReportsParameterLine, List<RegionLine> ADinamicRegionReportList)
        {
           /* string lStartTime = DateTime.Now.ToString(); //для оптимизации
            string lEndTime;*/
            List<DinamicMetricReportLine> lMetricReportList = new List<DinamicMetricReportLine>();
            if (AReportsParameterLine.ReportParameterID > 0)
            {
                string lSQL;
                lSQL = GetMetricReportListFullSQL(AReportsParameterLine, ADinamicRegionReportList);
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllMetricReportListCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = lSQL,
                            CommandType = CommandType.Text
                        };

                        connection.Open();
                        using (SqlDataReader MetricReportReader = selectAllMetricReportListCommand.ExecuteReader())
                        {   // получили полный запрос значений
                            while (MetricReportReader.Read())
                            {   //получили объект строки показателя отчета
                                DinamicMetricReportLine currentMetricReport =
                                    new DinamicMetricReportLine(long.Parse(MetricReportReader["MetricID"].ToString()),
                                                                MetricReportReader["MetricName"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricName"],
                                                                MetricReportReader["MetricCode"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricCode"],
                                                                MetricReportReader["MetricSort"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricSort"],
                                                                MetricReportReader["MetricScreenCode"] == DBNull.Value ? string.Empty : (string)MetricReportReader["MetricScreenCode"],
                                                                MetricReportReader["MetricFlag"] == DBNull.Value ? 0 : (long)long.Parse(MetricReportReader["MetricFlag"].ToString())
                                                                );
                                // теперь необходимо получить объекты значений региональных дирекций и пришить к показателю
                                foreach (RegionLine lRegionLine in ADinamicRegionReportList)
                                {
                                    currentMetricReport.MetricRegionFactList.Add(GetOnlyOneFactRowLine(currentMetricReport, lRegionLine, MetricReportReader));
                                }
                                lMetricReportList.Add(currentMetricReport); // добавили его в лист показателей уже польностью готовый
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }

             /*lEndTime = DateTime.Now.ToString();  // для оптимизации 0 сек отрабатывает процедура на пустой таблице Fact
             lEndTime = lStartTime + "  |  " + lEndTime; */
            return lMetricReportList;
        }
        public static string GetMetricReportListFullSQL(ReportsParameterLine AReportsParameterLine, List<RegionLine> ADinamicRegionReportList)
        {   
            string lResultSQL = "";
            string lSectionSQL;
            if (ADinamicRegionReportList.Count > 0) 
            {
                lResultSQL = KROSQLs.DinamicSQLSelectFactSectionSelect;
                foreach (RegionLine lRegionLine in ADinamicRegionReportList)
                {
                    lSectionSQL = KROSQLs.DinamicSQLSelectFactSectionAddRegionSelect.Replace("@Number", lRegionLine.RegionID.ToString());
                    lResultSQL = lResultSQL + lSectionSQL;
                }
                lResultSQL = lResultSQL + KROSQLs.DinamicSQLSelectFactSectionFrom;
                foreach (RegionLine lRegionLine in ADinamicRegionReportList)
                {
                    lSectionSQL = KROSQLs.DinamicSQLSelectFactSectionAddRegionLeftJoin.Replace("@Number", lRegionLine.RegionID.ToString());
                    lSectionSQL = lSectionSQL.Replace("@DepartmentID", AReportsParameterLine.DepartmentID.ToString());
                    lSectionSQL = lSectionSQL.Replace("@ContractorID", AReportsParameterLine.ContractorID.ToString());
                    lSectionSQL = lSectionSQL.Replace("@MonthID", AReportsParameterLine.MonthID.ToString());
                    lSectionSQL = lSectionSQL.Replace("@YearID", AReportsParameterLine.YearID.ToString());
                    lSectionSQL = lSectionSQL.Replace("@TypeID", AReportsParameterLine.TypeID.ToString());
                    lResultSQL = lResultSQL + lSectionSQL;
                }
                lResultSQL = lResultSQL + KROSQLs.DinamicSQLSelectFactSectionWhere.Replace("@ReportParameterID", AReportsParameterLine.ReportParameterID.ToString());
                lResultSQL = lResultSQL + KROSQLs.DinamicSQLSelectFactSectionOrder;
            } 
            else 
            {
                lResultSQL = KROSQLs.DinamicSQLSelectFactSectionSelect +
                             KROSQLs.DinamicSQLSelectFactSectionFrom +
                             KROSQLs.DinamicSQLSelectFactSectionWhere.Replace("@ReportParameterID", AReportsParameterLine.ReportParameterID.ToString()) +
                             KROSQLs.DinamicSQLSelectFactSectionOrder;
            }
            return lResultSQL;
        }
        public static DinamicFactLine GetOnlyOneFactRowLine(DinamicMetricReportLine AMetricReportLine, RegionLine ARegionLine, SqlDataReader AReportReader)
        {
            // все проверки опускаем ответственность до старта голбальной выборки данных в GetMetricReportList
            DinamicFactLine lResult = new DinamicFactLine(ARegionLine.RegionID);
            if ((AMetricReportLine.MetricID > 0) && (!AMetricReportLine.FlagMetricTextRow))  //ищем значения показателей в базе только не для  текстовых полей
            {   
                string lRegionID = ARegionLine.RegionID.ToString();
                lResult.FactID = AReportReader["FactID"+lRegionID] == DBNull.Value ? null : (long?)long.Parse(AReportReader["FactID"+lRegionID].ToString());
                if (lResult.FactID != null) { // попытаемся сократить  время на обработку
                    lResult.FactYearPlan       = AReportReader["FactYearPlan" + lRegionID] == DBNull.Value ? null : (double?)double.Parse(AReportReader["FactYearPlan" + lRegionID].ToString(), CultureInfo.CurrentCulture);
                    lResult.FactCumulativePlan = AReportReader["FactCumulativePlan" + lRegionID] == DBNull.Value ? null : (double?)double.Parse(AReportReader["FactCumulativePlan" + lRegionID].ToString(), CultureInfo.CurrentCulture);
                    lResult.FactMade           = AReportReader["FactMade" + lRegionID] == DBNull.Value ? null : (double?)double.Parse(AReportReader["FactMade" + lRegionID].ToString(), CultureInfo.CurrentCulture);
                }
            }
            return lResult;
        }
        public static DinamicFactLine SelectOnlyOneFactRow(ReportsParameterLine AReportsParameterLine, DinamicMetricReportLine AMetricReportLine, RegionLine ARegionLine)
        {   // все проверки опускаем ответственность до старта голбальной выборки данных в GetMetricReportList
            DinamicFactLine lResult = new DinamicFactLine(ARegionLine.RegionID);
            if ((AMetricReportLine.MetricID > 0) && (!AMetricReportLine.FlagMetricTextRow))  //ищем значения показателей в базе только не для  текстовых полей
            {
                using (
                    SqlConnection AConnection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                  try 
                  {
                    SqlCommand SelectOnlyOneFactRowCommand = new SqlCommand
                    {
                        Connection = AConnection,
                        CommandText = KROSQLs.SELECT_ONLYONEFACTROW_SP_NAME,
                        CommandType = CommandType.StoredProcedure
                    };

                    SqlParameter lParameter = null;

                    lParameter = new SqlParameter("@RegionID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = ARegionLine.RegionID;
                    SelectOnlyOneFactRowCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@MetricID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AMetricReportLine.MetricID;
                    SelectOnlyOneFactRowCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@DepartmentID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportsParameterLine.DepartmentID;
                    SelectOnlyOneFactRowCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@ContractorID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportsParameterLine.ContractorID;
                    SelectOnlyOneFactRowCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@MonthID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportsParameterLine.MonthID;
                    SelectOnlyOneFactRowCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@YearID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportsParameterLine.YearID;
                    SelectOnlyOneFactRowCommand.Parameters.Add(lParameter);

                    lParameter = new SqlParameter("@TypeID", SqlDbType.Int);
                    lParameter.IsNullable = false;
                    lParameter.Value = AReportsParameterLine.TypeID;
                    SelectOnlyOneFactRowCommand.Parameters.Add(lParameter);

                    AConnection.Open();
                    using (SqlDataReader FactRowReader = SelectOnlyOneFactRowCommand.ExecuteReader())
                    {
                        if (FactRowReader.Read())
                        {
                            lResult.FactID             = FactRowReader["ID"] == DBNull.Value ? null : (long?)long.Parse(FactRowReader["ID"].ToString());
                            lResult.FactYearPlan       = FactRowReader["YearPlan"] == DBNull.Value ? null : (double?)double.Parse(FactRowReader["YearPlan"].ToString(), CultureInfo.CurrentCulture);
                            lResult.FactCumulativePlan = FactRowReader["CumulativePlan"] == DBNull.Value ? null : (double?)double.Parse(FactRowReader["CumulativePlan"].ToString(), CultureInfo.CurrentCulture);
                            lResult.FactMade           = FactRowReader["Made"] == DBNull.Value ? null : (double?)double.Parse(FactRowReader["Made"].ToString(), CultureInfo.CurrentCulture);
                        }
                    }
                } finally { if (AConnection != null) AConnection.Close(); }
            }
        }
            return lResult;
        } 
        public static List<MetricFunctionLine> GetMetricFunctionListByMetricListInPeriodID(int AMetricListInPeriodID)
        {   // получается что данный запрос возвращает только выбранные показатели в формуле поэтому при формировании List checked выставляем в true
            List<MetricFunctionLine> lMetricFunctionList = new List<MetricFunctionLine>();
            if (AMetricListInPeriodID > 0)  // только при наличии входных параметров отчета
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllMetricFunctionListCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_METRICFUNCTIONLIST_BY_METRICLISTINPERIODID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@MetricListInPeriodID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AMetricListInPeriodID;
                        selectAllMetricFunctionListCommand.Parameters.Add(lParameter);

                        connection.Open();
                        using (SqlDataReader MetricFunctionListReader = selectAllMetricFunctionListCommand.ExecuteReader())
                        {
                            while (MetricFunctionListReader.Read())
                            {
                                MetricFunctionLine currentRole =
                                new MetricFunctionLine(int.Parse(MetricFunctionListReader["MetricID"].ToString()),
                                                                MetricFunctionListReader["SumMetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionListReader["SumMetricID"].ToString()),
                                                                MetricFunctionListReader["DK"] == DBNull.Value ? 1 : (int)int.Parse(MetricFunctionListReader["DK"].ToString())
                                                                );
                                lMetricFunctionList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lMetricFunctionList;
        }
        public static List<MetricFunctionLine> GetMetricFunctionListByReportParameterID(int AReportParameterID)
        {   // получается что данный запрос возвращает только выбранные показатели в формуле поэтому при формировании List checked выставляем в true
            List<MetricFunctionLine> lMetricFunctionList = new List<MetricFunctionLine>();
            if (AReportParameterID > 0)  // только при наличии входных параметров отчета
            {
                using (
                    SqlConnection connection =
                        new SqlConnection(ConfigurationManager.ConnectionStrings["EFDbContext"].ConnectionString))
                {
                    try
                    {
                        SqlCommand selectAllMetricFunctionListCommand = new SqlCommand
                        {
                            Connection = connection,
                            CommandText = KROSQLs.SELECT_ALL_METRICFUNCTIONLIST_BY_REPORTPARAMETERID_SP_NAME,
                            CommandType = CommandType.StoredProcedure
                        };
                        SqlParameter lParameter = null;
                        lParameter = new SqlParameter("@ReportPatrameterID", SqlDbType.Int);
                        lParameter.IsNullable = false;
                        lParameter.Value = AReportParameterID;
                        selectAllMetricFunctionListCommand.Parameters.Add(lParameter);

                        connection.Open();
                        using (SqlDataReader MetricFunctionListReader = selectAllMetricFunctionListCommand.ExecuteReader())
                        {
                            while (MetricFunctionListReader.Read())
                            {
                                MetricFunctionLine currentRole =
                                new MetricFunctionLine(int.Parse(MetricFunctionListReader["MetricID"].ToString()),
                                                                MetricFunctionListReader["SumMetricID"] == DBNull.Value ? -1 : (int)int.Parse(MetricFunctionListReader["SumMetricID"].ToString()),
                                                                MetricFunctionListReader["DK"] == DBNull.Value ? 1 : (int)int.Parse(MetricFunctionListReader["DK"].ToString())
                                                                );
                                lMetricFunctionList.Add(currentRole);
                            }
                        }
                    }
                    finally { if (connection != null) connection.Close(); }
                }
            }
            return lMetricFunctionList;
        }
        # endregion
    }
}