﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using KRO.Authentication;
using KRO.Models;
using KRO.Contents;

namespace KRO.Pages.Admin
{
    public partial class ChangeUser : System.Web.UI.Page
    {
        public User FChangeUser {
            get { return (User)ViewState["CheckedUser"]; }
            set { ViewState["CheckedUser"] = value; }
        }
        protected void Page_Load(object sender, EventArgs e)
        {
            

            if (!Page.IsPostBack)
            {
                ddlDepartment.DataSource = ModelsManager.GetDepartmentList();
                ddlDepartment.DataBind();
                ddlRole.DataSource = ModelsManager.GetRolesList();
                ddlRole.DataBind();
                Users prevPage = PreviousPage as Users;
                if (prevPage != null)
                    FChangeUser = prevPage.CheckedUser;
                InitFieldUser(); 
            }
        }

        private void InitFieldUser()
        {
            if (FChangeUser == null) FChangeUser = new User();
            tbFamily.Text = FChangeUser.Family;
            tbFirstName.Text = FChangeUser.FirstName;
            tbSecondName.Text = FChangeUser.SecondName;
            tbLogin.Text = FChangeUser.Name;
            ddlDepartment.SelectedValue = FChangeUser.UserDepartmentID.ToString();
            ddlRole.SelectedValue = FChangeUser.UserRoleID.ToString();
            if ((FChangeUser.UserFlag & KROFlag.flUserCheckAutoPassword) == KROFlag.flUserCheckAutoPassword)
            {
                cbCheckAutoPassword.Checked = true;
                tbPassword.Text = FChangeUser.Password; //администратор может видеть только временные пароли (требования заказчика)
            }
            else
            {
                cbCheckAutoPassword.Checked = false;
                tbPassword.Text = FChangeUser.Password; // требования изменились видят пароли всегда
            }
            if ((FChangeUser.UserFlag & KROFlag.flUserAddNewReportParameter) == KROFlag.flUserAddNewReportParameter) cbCheckAddReportParameter.Checked = true; else cbCheckAddReportParameter.Checked = false;
            if ((FChangeUser.UserFlag & KROFlag.flUserRCKU) == KROFlag.flUserRCKU) cbCheckRCKU.Checked = true; else cbCheckRCKU.Checked = false;
            if ((FChangeUser.UserFlag & KROFlag.flUserRegionDirection) == KROFlag.flUserRegionDirection) rbRule.SelectedIndex = 2;
            if ((FChangeUser.UserFlag & KROFlag.flUserFilialDepartment) == KROFlag.flUserFilialDepartment) rbRule.SelectedIndex = 1;
            if ((FChangeUser.UserFlag & KROFlag.flUserEconomicDepartment) == KROFlag.flUserEconomicDepartment) rbRule.SelectedIndex = 0;

            tbOfficeName.Text = FChangeUser.OfficeName;
            tbPhone.Text = FChangeUser.Phone;
            tbMail.Text = FChangeUser.EMail;
        }

        protected void ButtonCancel_Click(object sender, EventArgs e)
        {
            Server.Transfer(Constants.USERS_PAGE_URL);
        }
        protected void ButtonApply_Click(object sender, EventArgs e)
        {
            lblExecuteError.Visible = false;
            bool lResultExecute = false;
            if (FChangeUser == null) FChangeUser = new User();
            FChangeUser.Family = tbFamily.Text;
            FChangeUser.FirstName = tbFirstName.Text;
            FChangeUser.SecondName = tbSecondName.Text;
            FChangeUser.OfficeName = tbOfficeName.Text;
            FChangeUser.Phone = tbPhone.Text;
            FChangeUser.EMail = tbMail.Text;
            FChangeUser.Name = tbLogin.Text;
            FChangeUser.Password = tbPassword.Text; 
            FChangeUser.UserRoleID = Int32.Parse(ddlRole.SelectedValue);
            FChangeUser.UserRoleName = ddlRole.Text;
            FChangeUser.UserDepartmentID = Int32.Parse(ddlDepartment.SelectedValue);
            FChangeUser.UserDepartmentName = ddlDepartment.Text;
            //определим дату действия пароля
            if (cbCheckAutoPassword.Checked)
            {   //выставлен временный пароль обязательно обновляем дату
                FChangeUser.DatePassword = DateTime.Now;
            }
            else
            {  // пароль не временный значит идет обновление данных либо добавление нового пользователя.
                //если пользователь новый или дата отсутствует установим текущую дату 
                if ((FChangeUser.ID == -1) || (FChangeUser.DatePassword == null)) FChangeUser.DatePassword = DateTime.Now;                                
            }
            //соберем флаги
            long lFlag = 0;
            if (cbCheckAutoPassword.Checked) lFlag = lFlag |= KROFlag.flUserCheckAutoPassword;
            if (cbCheckAddReportParameter.Checked) lFlag = lFlag |= KROFlag.flUserAddNewReportParameter;
            if (cbCheckRCKU.Checked)
            {  // требование Бойцова от 7.10.15  если РЦКУ то не может быть работником ЦА
                lFlag = lFlag |= KROFlag.flUserRCKU;
                if ((rbRule != null) && (rbRule.SelectedValue.Length > 0) && (long.Parse(rbRule.SelectedValue) == 4))
                {  // сдесь меняем на филиал
                    rbRule.SelectedIndex = 1;
                }
            }   
            // требование Бойцова от 7.10.15 в противном случае оставляем кого выбрали
           if ((rbRule != null) && (rbRule.SelectedValue.Length > 0)) lFlag = lFlag |= long.Parse(rbRule.SelectedValue);
                            
            FChangeUser.UserFlag = lFlag;
            //конец сбора флагов
            int lUserID = FChangeUser.ID; //переменная необходима для проверки перед выводом в LogOut. а что собственно пытались сделать
            //попытка сохранения результатов и соответствующая реакция
            try
            {
                if (FChangeUser.ID > 0) lResultExecute = UsersManager.UpdateUser(FChangeUser);
                else lResultExecute = UsersManager.InsertUser(FChangeUser);
                if (!lResultExecute) lblExecuteError.Text = KROConstants.msgUserUpdateError; //восстановим сообщение мало-ли до этого менялось
            }
            catch (Exception exp)
            {   //определим что за байда приключилась
                if (exp.Message.IndexOf(KROConstants.fndExcpUserIndexError) > -1) lblExecuteError.Text = KROConstants.msgUserIndexError;
                else lblExecuteError.Text = KROConstants.msgUserUpdateError;
            }
            // обработка результатов сохранения
            if (lResultExecute)
            {
                // сохраним удачное сообщение в LogOuts
                if (lUserID > 0) ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgUpdateUser + " | " + FChangeUser.Name + " | " + FChangeUser.ID, 3), KROFlag.flLogExchangeUsers, AuthenticationManager.GetAuthenticationUserModel(Session).ID);
                else ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgInsertUser + " | " + FChangeUser.Name + " | " + FChangeUser.ID, 4), KROFlag.flLogExchangeUsers, AuthenticationManager.GetAuthenticationUserModel(Session).ID);
                Server.Transfer(Constants.USERS_PAGE_URL);
            }
            else
            {
                // сохраним неудачное сообщение в LogOuts
                if (lUserID > 0) ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgUpdateUser + " | " + FChangeUser.Name + " | " + FChangeUser.ID, 2), KROFlag.flLogExchangeUsers, AuthenticationManager.GetAuthenticationUserModel(Session).ID);
                else ModelsManager.InsertLogOut(ModelsManager.GetLogMessage(KROConstants.lgInsertUser + " | " + FChangeUser.Name + " | " + FChangeUser.ID, 2), KROFlag.flLogExchangeUsers, AuthenticationManager.GetAuthenticationUserModel(Session).ID);
                lblExecuteError.Visible = true;
            }
        }
    }
}